import cmd2
from typing import (
    Any,
    Callable,
    Dict,
    Iterable,
    List,
    Mapping,
    Optional,
    Tuple,
    Type,
    Union,
)

from GLXShell.libs.properties.application import GLXShPropertyApplication
from GLXShell.libs.properties.shortcuts import GLXShPropertyShortcuts
from GLXShell.libs.properties.history import GLXShPropertyHistory
from GLXShell.libs.properties.config import GLXShPropertyConfig
from GLXShell.libs.properties.shell import GLXShPropertyShell

from GLXShell.libs.plugins import GLXShPluginsManager
from GLXShell.libs.intro import GLXShIntro
from GLXShell.libs.prompt import GLXShPrompt
from GLXShell.libs.settable import GLXShSettable


# Key binding
# http://readline.kablamo.org/emacs.html


class GLXShell(
    GLXShPropertyApplication,
    GLXShPropertyShortcuts,
    GLXShPropertyHistory,
    GLXShPropertyConfig,
    GLXShPropertyShell,
    GLXShPluginsManager,
    GLXShIntro,
    GLXShPrompt,
    cmd2.Cmd,
    GLXShSettable,
):
    """An easy but powerful survival kit for writing line-oriented command interpreters.

    Extends the Python Standard Library’s cmd2 package by adding a lot of useful features
    to the out of the box configuration.

    Line-oriented command interpreters are often useful for test harnesses, internal tools, and rapid prototypes.

    :param completekey: readline name of a completion key, default to Tab
    :param stdin: alternate input file object, if not specified, sys.stdin is used
    :param stdout: alternate output file object, if not specified, sys.stdout is used
    :param persistent_history_file: file path to load a persistent cmd2 command history from
    :param persistent_history_length: max number of history items to write
                                      to the persistent history file
    :param startup_script: file path to a script to execute at startup
    :param use_ipython: should the "ipy" command be included for an embedded IPython shell
    :param allow_cli_args: if ``True``, then :meth:`cmd2.Cmd.__init__` will process command
                           line arguments as either commands to be run or, if ``-t`` or
                           ``--test`` are given, transcript files to run. This should be
                           set to ``False`` if your application parses its own command line
                           arguments.
    :param transcript_files: pass a list of transcript files to be run on initialization.
                             This allows running transcript tests when ``allow_cli_args``
                             is ``False``. If ``allow_cli_args`` is ``True`` this parameter
                             is ignored.
    :param allow_redirection: If ``False``, prevent output redirection and piping to shell
                              commands. This parameter prevents redirection and piping, but
                              does not alter parsing behavior. A user can still type
                              redirection and piping tokens, and they will be parsed as such
                              but they won't do anything.
    :param multiline_commands: list of commands allowed to accept multi-line input
    :param terminators: list of characters that terminate a command. These are mainly
                        intended for terminating multiline commands, but will also
                        terminate single-line commands. If not supplied, the default
                        is a semicolon. If your app only contains single-line commands
                        and you want terminators to be treated as literals by the parser,
                        then set this to an empty list.
    :param shortcuts: dictionary containing shortcuts for commands. If not supplied,
                      then defaults to constants.DEFAULT_SHORTCUTS. If you do not want
                      any shortcuts, pass an empty dictionary.
    :param command_sets: Provide CommandSet instances to load during cmd2 initialization.
                         This allows CommandSets with custom constructor parameters to be
                         loaded.  This also allows the a set of CommandSets to be provided
                         when `auto_load_commands` is set to False
    :param auto_load_commands: If True, cmd2 will check for all subclasses of `CommandSet`
                               that are currently loaded by Python and automatically
                               instantiate and register all commands. If False, CommandSets
                               must be manually installed with `register_command_set`.
    :param auto_load_plugins: If True, plugins will be load at end of the init
    :param auto_load_settable: If True, settable will be load at end of the init
    :param default_to_shell: If True, unknowns commands will be sed to the sub system
    """

    def __init__(
        self,
        completekey: str = "tab",
        stdin=None,
        stdout=None,
        *,
        persistent_history_file: str = "",
        persistent_history_length: int = 1000,
        startup_script: str = "",
        allow_cli_args: bool = True,
        transcript_files: Optional[List[str]] = None,
        allow_redirection: bool = True,
        multiline_commands: Optional[List[str]] = None,
        terminators: Optional[List[str]] = None,
        shortcuts: Optional[Dict[str, str]] = None,
        command_sets: Optional[Iterable[cmd2.CommandSet]] = None,
        auto_load_commands: bool = False,
        auto_load_plugins: bool = True,
        auto_load_settable: bool = True,
        default_to_shell: bool = True,
        debug: bool = False,
    ) -> None:
        GLXShPropertyApplication.__init__(self)
        GLXShPropertyShortcuts.__init__(self)
        GLXShPropertyHistory.__init__(self)
        GLXShPropertyShell.__init__(self)
        GLXShPropertyConfig.__init__(self)

        self.shell = self

        GLXShIntro.__init__(self)
        GLXShPrompt.__init__(self)
        GLXShPluginsManager.__init__(self)

        self.persistent_history_file = persistent_history_file
        self.persistent_history_length = persistent_history_length
        self.shortcuts = shortcuts

        # Init and config the cmd2 object
        cmd2.Cmd.__init__(
            self,
            completekey=completekey,
            stdin=stdin,
            stdout=stdout,
            persistent_history_file=self.persistent_history_file,
            persistent_history_length=self.persistent_history_length,
            startup_script=startup_script,
            allow_cli_args=allow_cli_args,
            transcript_files=transcript_files,
            allow_redirection=allow_redirection,
            multiline_commands=multiline_commands,
            terminators=terminators,
            shortcuts=self.shortcuts,
            command_sets=command_sets,
            auto_load_commands=auto_load_commands,
        )
        self.debug = debug
        self.default_to_shell = default_to_shell
        self.auto_load_settable = auto_load_settable
        self.auto_load_plugins = auto_load_plugins
        # PostProcessing
        if self.auto_load_plugins:
            self.plugins_manager_control_config()
            self.load_plugins()
        if self.auto_load_settable:
            GLXShSettable(self).load_settable()

    @property
    def intro(self):
        """
        Return intro to displays, it property is call one time at startup by ``cmd2`` python module
        """
        return self.intro_to_display

    @property
    def prompt(self):
        """
        Return prompt to displays, the prompt is display just before the blinking cursor on the terminal
        """
        return self.prompt_to_display



import os
import cmd2
import argparse

from GLXShell.plugins.builtins import PLUGIN_VERSION
from GLXShell.plugins.builtins import PLUGIN_DESCRIPTION
from GLXShell.plugins.builtins import PLUGIN_WARRANTY

which_parser = cmd2.Cmd2ArgumentParser(
    description="which  returns  the pathnames of the files (or links) which would be executed in the current "
    "environment, had its arguments been given as commands in a strictly POSIX-conformant shell.  It does "
    "this by searching the PATH for executable files matching the names  of  the arguments. It does not "
    "canonicalize path names. "
)
which_parser.add_argument(
    "-a",
    "--all",
    action="store_true",
    help="print all matching pathnames of each argument",
)

which_parser.add_argument(
    "--version",
    action="store_true",
    help="output version information and exit",
)

which_parser.add_argument(
    "filename",
    nargs="?",
    const=1,
)


class GLXWhich(cmd2.CommandSet):
    def __init__(self):
        super().__init__()

    @staticmethod
    def which_print_version():
        cmd2.Cmd().poutput(
            "which ({name}) v{version}\n{licence}".format(
                name=PLUGIN_DESCRIPTION,
                version=PLUGIN_VERSION,
                licence=PLUGIN_WARRANTY,
            )
        )

    @staticmethod
    def which(all_info=False, filename=None):
        for exec_path in os.get_exec_path():
            if os.path.isfile(os.path.join(exec_path, filename)):
                if os.access(os.path.join(exec_path, filename), os.X_OK):
                    cmd2.Cmd().poutput(os.path.join(exec_path, filename))
                    if not all_info:
                        return

    @cmd2.with_argparser(which_parser)
    @cmd2.with_category("Builtins")
    def do_which(self, args):
        if args.version:  # pragma: no cover
            self.which_print_version()
            return

        if args.filename is None:  # pragma: no cover
            which_parser.print_usage()
            return

        self.which(all_info=args.all, filename=args.filename)  # pragma: no cover



from GLXShell.libs.plugins import GLXShPluginsManager


class GLXShellPropertyPluginsManager(object):
    def __init__(self):
        self.__plugins_manager = None
        self.plugins_manager = None

    @property
    def plugins_manager(self):
        """
        ``plugins_manager`` property store the plugins_manager object and assure to set plugins_manager value only if
         have changed

        :return: a Galaxie Plugins Manager instance or None
        :rtype: GLXShPluginsManager or None
        """
        return self.__plugins_manager

    @plugins_manager.setter
    def plugins_manager(self, value=None):
        """
        Set ``plugins_manager`` property value

        :param value: a Galaxie Plugins Manager instance or None
        :type value: GLXShPluginsManager or None
        :raise TypeError: when property value is not a Galaxie Plugins Manager instance or None
        """
        if value is None:
            value = GLXShPluginsManager(shell=self)
        if value is not None and not isinstance(value, GLXShPluginsManager):
            raise TypeError(
                "'plugins_manager' property value must be a Galaxie Plugins Manager instance or None"
            )
        if self.plugins_manager != value:
            self.__plugins_manager = value



# http://pwet.fr/man/linux/commandes/posix/touch/
# https://www.maizure.org/projects/decoded-gnu-coreutils/touch.html
# https://pubs.opengroup.org/onlinepubs/9699919799/utilities/touch.html
import argparse

import cmd2

from GLXShell.plugins.builtins import PLUGIN_DESCRIPTION
from GLXShell.plugins.builtins import PLUGIN_VERSION
from GLXShell.plugins.builtins import PLUGIN_WARRANTY

touch_parser = argparse.ArgumentParser(
    description="The touch utility shall change the modification times, access times, or both of files. "
                "The modification time shall be equivalent to the value of the st_mtime member of the stat "
                " structure for a file, as described in the System Interfaces volume of IEEE Std 1003.1-2001; "
                "the access time shall be equivalent to the value of st_atime."
                ""
                "The time used can be specified by the -t time option-argument, the corresponding time fields "
                "of the file referenced by the -r ref_file option-argument, or the date_time operand, as specified "
                "in the following sections. If none of these are specified, touch shall use the current time "
                "(the value returned by the equivalent of the time() function defined in the System Interfaces "
                "volume of IEEE Std 1003.1-2001).")
#      --help               display this help and exit
#      --version            output version information and exit
touch_parser.add_argument(
    "--version", action="store_true", help="output version information and exit"
)
touch_parser.add_argument(
    "-a",
    dest="access_time",
    help="Change the access time of file. Do not change the modification time unless -m is also specified.",
)
touch_parser.add_argument(
    "-c",
    help="Do not create a specified file if it does not exist. Do not write any diagnostic messages "
         "concerning this condition.",
)
touch_parser.add_argument(
    "-m",
    help="Change the modification time of file. Do not change the access time unless -a is also specified.",
)
touch_parser.add_argument(
    "-r",
    help="Use the corresponding time of the file named by the pathname ref_file instead of the current time.",
)
touch_parser.add_argument(
    "-t",

    help="Use the specified time instead of the current time. The option-argument shall be a decimal number of the form",
)

touch_parser.add_argument(
    "file",
    help="Use the specified time instead of the current time. The option-argument shall be a decimal number of the form"

)


class GLXTouch(cmd2.CommandSet):
    def __init__(self):
        super().__init__()
        # Internal Variables

    @property
    def result(self):
        return True

    @staticmethod
    def touch_print_version():
        cmd2.Cmd().poutput(
            "touch ({name}) v{version}\n{licence}".format(
                name=PLUGIN_DESCRIPTION,
                version=PLUGIN_VERSION,
                licence=PLUGIN_WARRANTY,
            )
        )

    def touch(self, file=None):
        raise NotImplementedError("Touch is not implemented yet ..., use !touch")

    @cmd2.with_argparser(touch_parser)
    @cmd2.with_category("Builtins")
    def do_touch(self, args):
        # print(args.NUMBER)
        if args.version:  # pragma: no cover
            self.touch_print_version()
            return

        if args.file:  # pragma: no cover
            self.touch(file=args.file)
        else:
            touch_parser.print_help()  # pragma: no cover



import os
import cmd2
import argparse
import sys

# https://www.maizure.org/projects/decoded-gnu-coreutils/
# https://www.maizure.org/projects/decoded-gnu-coreutils/cat.html

from GLXShell.plugins.builtins import PLUGIN_VERSION
from GLXShell.plugins.builtins import PLUGIN_DESCRIPTION
from GLXShell.plugins.builtins import PLUGIN_WARRANTY

cat_parser = argparse.ArgumentParser()
cat_parser.add_argument(
    "file",
    nargs="*",
    type=argparse.FileType("r"),
    default=sys.stdin,
    help="with no FILE, or when FILE is -, read standard input.",
    completer=cmd2.Cmd.path_complete
)
cat_parser.add_argument(
    "-A",
    "--show-all",
    action="store_true",
    help="equivalent to -vET",
)
cat_parser.add_argument(
    "-b",
    "--number-nonblank",
    action="store_true",
    help="number nonempty output lines, overrides -n",
)
cat_parser.add_argument(
    "-e",
    action="store_true",
    help="equivalent to -vE",
)
cat_parser.add_argument(
    "-E",
    "--show-ends",
    action="store_true",
    help="display $ at end of each line",
)
cat_parser.add_argument(
    "-n",
    "--number",
    action="store_true",
    help="number all output lines",
)
cat_parser.add_argument(
    "-s",
    "--squeeze-blank",
    action="store_true",
    help="suppress repeated empty output lines",
)
#   -t                       equivalent to -vT
cat_parser.add_argument(
    "-t",
    action="store_true",
    help="equivalent to -vT",
)
#   -T, --show-tabs          display TAB characters as ^I
cat_parser.add_argument(
    "-T",
    "--show-tabs",
    action="store_true",
    help="display TAB characters as ^I",
)
#   -u                       (ignored)
cat_parser.add_argument(
    "-u",
    action="store_true",
    help="(ignored)",
)
#   -v, --show-nonprinting   use ^ and M- notation, except for LFD and TAB
cat_parser.add_argument(
    "-v",
    "--show-nonprinting",
    action="store_true",
    help="use ^ and M- notation, except for LFD and TAB",
)
#      --help               display this help and exit
#      --version            output version information and exit
cat_parser.add_argument(
    "--version", action="store_true", help="output version information and exit"
)


class GLXCat(cmd2.CommandSet):
    def __init__(self, *args, **kwargs):
        super().__init__()
        self.stdout = kwargs.get("stdout", sys.stdout)
        self.stdin = kwargs.get("stdin", sys.stdin)
        self.stderr = kwargs.get("stdin", sys.stderr)

    @staticmethod
    def cat_print_version():
        cmd2.Cmd().poutput(
            "cat ({name}) v{version}\n{licence}".format(
                name=PLUGIN_DESCRIPTION,
                version=PLUGIN_VERSION,
                licence=PLUGIN_WARRANTY,
            )
        )

    @staticmethod
    def cat(file, number_nonblank, number, show_tabs, squeeze_blank, show_ends):
        if number_nonblank:
            number = False

        file_data = []
        for file_object in file:
            file_data.append(file_object.read().split("\n"))

        think_to_display = []
        line_number = 1
        blank_line_allowed = True

        for file in file_data:
            if not blank_line_allowed:
                blank_line_allowed = True

            for line in file:
                if show_tabs:
                    line = line.replace("\t", "^I")
                if number:
                    if len(line) > 0:
                        think_to_display.append(
                            "{0:>6d}  {1}".format(line_number, line)
                        )
                        line_number += 1
                        if not blank_line_allowed:
                            blank_line_allowed = True
                    else:
                        if blank_line_allowed:
                            think_to_display.append(
                                "{0:>6d}  {1}".format(line_number, "")
                            )
                            line_number += 1
                            if squeeze_blank and blank_line_allowed:
                                blank_line_allowed = False
                elif number_nonblank:
                    if len(line) > 0:
                        think_to_display.append(
                            "{0:>6d}  {1}".format(line_number, line)
                        )
                        line_number += 1
                        if not blank_line_allowed:
                            blank_line_allowed = True
                    else:
                        if blank_line_allowed:
                            think_to_display.append("")
                            if squeeze_blank and blank_line_allowed:
                                blank_line_allowed = False
                else:
                    if len(line) > 0:
                        think_to_display.append(line)
                        if not blank_line_allowed:
                            blank_line_allowed = True
                    else:
                        if blank_line_allowed:
                            think_to_display.append(line)
                            if squeeze_blank and blank_line_allowed:
                                blank_line_allowed = False

        if show_ends:
            last_result = "$\n".join(think_to_display)
        else:
            last_result = "\n".join(think_to_display)

        cmd2.Cmd().last_result = last_result
        cmd2.Cmd().poutput(msg=last_result, end="")

    @cmd2.with_argparser(cat_parser)
    @cmd2.with_category("Builtins")
    def do_cat(self, args):
        """Concatenate FILE(s) to standard output"""
        if args.version:  # pragma: no cover
            self.cat_print_version()
            return

        self.cat(
            file=args.file,
            number_nonblank=args.number_nonblank,
            number=args.number,
            show_tabs=args.show_tabs,
            squeeze_blank=args.squeeze_blank,
            show_ends=args.show_ends,
        )  # pragma: no cover



class GLXShPropertyShell(object):
    def __init__(self):
        self.__shell = None
        self.shell = None

    @property
    def shell(self):
        """
        ``shell`` property store the shell object and assure to set shell value only if have changed

        :return: a Galaxie Shell instance or None
        :rtype: GLXShell.Shell or None
        """
        return self.__shell

    @shell.setter
    def shell(self, value=None):
        """
        Set ``shell`` property value

        :param value: a Galaxie Shell instance or None
        :type value: GLXShell.Shell or None
        :raise TypeError: when property value is not a Galaxie Shell instance or None
        """
        # if value is None:
        #     value = self
        # if value is not None and not isinstance(value, GLXShell()):
        #     raise TypeError("'shell' property value must be a Galaxie Shell instance or None")
        if self.shell != value:
            self.__shell = value



import cmd2


class GLXShPropertyShortcuts(object):
    def __init__(self):
        self.__shortcuts = None
        self.shortcuts = dict(cmd2.DEFAULT_SHORTCUTS)

    @property
    def shortcuts(self):
        """
        Store the shortcuts

        :return: dictionary containing shortcuts for commands. If not supplied,
                 then defaults to cmd2.DEFAULT_SHORTCUTS. If you do not want
                 any shortcuts, pass an empty dictionary.
        :rtype: dict
        """
        return self.__shortcuts

    @shortcuts.setter
    def shortcuts(self, value=None):
        """
        Set the ``shortcuts`` property value

        Notes: ``None`` will restore default value cmd2.DEFAULT_SHORTCUTS

        :param value: dictionary containing shortcuts for commands. If not supplied,
                      then defaults to cmd2.DEFAULT_SHORTCUTS. If you do not want
                      any shortcuts, pass an empty dictionary
        :type value: dict or None
        """
        if value is None:
            value = {}
        if type(value) != dict:
            raise TypeError("'shortcuts' must be a dict type or None")
        if value != self.shortcuts:
            self.__shortcuts = value



PLUGIN_VERSION = "0.2.1"
PLUGIN_NAME = "builtins"
PLUGIN_DESCRIPTION = "glxsh builtins plugin"
PLUGIN_LICENSE = "GNU General Public License v3 or later (GPLv3+)"
PLUGIN_WARRANTY = """Copyright (C) 2020-2021 Galaxie Shell Project.
License GPLv3+: GNU GPL version 3 or later <https://gnu.org/licenses/gpl.html>.
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.
"""

from GLXShell.libs.commands import GLXShellCommands
from GLXShell.plugins.builtins.arch import GLXArch
from GLXShell.plugins.builtins.cat import GLXCat
from GLXShell.plugins.builtins.cd import GLXCd
from GLXShell.plugins.builtins.df import GLXDf
from GLXShell.plugins.builtins.dir import GLXDir
from GLXShell.plugins.builtins.mkdir import GLXMkdir
from GLXShell.plugins.builtins.pwd import GLXPwd
from GLXShell.plugins.builtins.rmdir import GLXRmDir
from GLXShell.plugins.builtins.sleep import GLXSleep
from GLXShell.plugins.builtins.touch import GLXTouch
from GLXShell.plugins.builtins.uname import GLXUname
from GLXShell.plugins.builtins.which import GLXWhich


class GLXShPluginBuiltins(GLXShellCommands):
    def __init__(self):
        GLXShellCommands.__init__(self)
        self.commands = [
            {"name": "arch", "object": GLXArch()},
            {"name": "cat", "object": GLXCat()},
            {"name": "cd", "object": GLXCd()},
            {"name": "df", "object": GLXDf()},
            {"name": "dir", "object": GLXDir()},
            {"name": "mkdir", "object": GLXMkdir()},
            {"name": "pwd", "object": GLXPwd()},
            {"name": "rmdir", "object": GLXRmDir()},
            {"name": "sleep", "object": GLXSleep()},
            {"name": "touch", "object": GLXTouch()},
            {"name": "uname", "object": GLXUname()},
            {"name": "which", "object": GLXWhich()},
        ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# It script it publish under GNU GENERAL PUBLIC APPLICATION_LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Shell Team, all rights reserved
import os
import sys
import argparse

# Require when you haven't GLXCurses as default Package
current_dir = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.dirname(current_dir))

from GLXShell.libs.shell import GLXShell


# https://fallout.fandom.com/wiki/Pip-OS_v7.1.0.8
# https://fallout.fandom.com/wiki/Terminal
# https://fallout.fandom.com/wiki/Unified_Operating_System
# https://fallout.fandom.com/wiki/RETROS_BIOS
# https://fallout.fandom.com/wiki/MF_Boot_Agent


def main(argv=None):
    """Run when invoked from the operating system shell"""

    glxsh_parser = argparse.ArgumentParser(description="Commands as arguments")
    glxsh_parser.add_argument(
        "command",
        nargs="?",
        help="optional commands or file to run, if no commands given, enter an interactive shell",
    )
    glxsh_parser.add_argument(
        "command_args",
        nargs=argparse.REMAINDER,
        help="if commands is not a file use optional arguments for commands",
    )

    args = glxsh_parser.parse_args(argv)

    shell = GLXShell(auto_load_plugins=True)

    sys_exit_code = 0
    if args.command:
        if os.path.isfile(args.command):
            # we have file to execute
            shell.onecmd_plus_hooks("@{command}".format(command=args.command))
        else:
            # we have a commands, run it and then exit
            shell.onecmd_plus_hooks(
                "{command} {args}".format(
                    command=args.command, args=" ".join(args.command_args)
                )
            )
    else:
        # we have no commands, drop into interactive mode
        sys_exit_code = shell.cmdloop()

    return sys_exit_code


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# It script it publish under GNU GENERAL PUBLIC APPLICATION_LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Shell Team, all rights reserved

import argparse
import cmd2

from GLXShell.libs.properties.plugins import GLXShPropertyPlugins
from GLXShell import APPLICATION_PLUGINS


def choices():
    to_return = []
    for plugin in APPLICATION_PLUGINS:
        if plugin and "name" in plugin:
            to_return.append(plugin["name"])
    return to_return


enable_parser = cmd2.Cmd2ArgumentParser()
enable_parser.add_argument("name", choices=choices())
disable_parser = cmd2.Cmd2ArgumentParser()
disable_parser.add_argument("name", choices=choices())
load_parser = cmd2.Cmd2ArgumentParser()
load_parser.add_argument("name", choices=choices())
unload_parser = cmd2.Cmd2ArgumentParser()
unload_parser.add_argument("name", choices=choices())
reload_parser = cmd2.Cmd2ArgumentParser()
reload_parser.add_argument("name", choices=choices())
plugins_parser = cmd2.Cmd2ArgumentParser(description="Plugins management")
plugins_subparsers = plugins_parser.add_subparsers(
    title="commands", help="a command it can be choose"
)


class GLXShPluginsManager(GLXShPropertyPlugins):
    def __init__(self, *args, **kwargs):
        GLXShPropertyPlugins.__init__(self)
        self.loaded_plugins = []

    def plugins_manager_control_config(self):
        if hasattr(self, "config") and self.config:
            have_change = False
            if "plugins" not in self.config.data:
                self.config.data["plugins"] = {}
                have_change = True
            for plugin in self.plugins:
                if plugin["name"] not in self.config.data["plugins"]:
                    self.config.data["plugins"][plugin["name"]] = {}
                    have_change = True
                if "enabled" not in self.config.data["plugins"][plugin["name"]]:
                    self.config.data["plugins"][plugin["name"]]["enabled"] = True
                    have_change = True
            if have_change:
                self.config.write_config()

    def load_plugins(self):
        self.plugins_manager_control_config()
        if (
            hasattr(self, "config")
            and self.config
            and hasattr(self, "shell")
            and self.shell
        ):
            for plugin in self.plugins:
                if self.config.data["plugins"][plugin["name"]]["enabled"]:
                    self.load(argparse.Namespace(name=plugin["name"]))

    @cmd2.as_subcommand_to("plugins", "enable", enable_parser)
    @cmd2.with_category("Plugins commands")
    def enable(self, ns: argparse.Namespace):
        """Enable at startup a plugin by it name and load it"""
        if (
            hasattr(self, "config")
            and self.config
            and hasattr(self, "shell")
            and self.shell
        ):
            for plugin in self.plugins:
                if ns.name == plugin["name"]:
                    if self.shell.debug:
                        self.shell.pwarning("ENABLE PLUGIN: {0}".format(plugin["name"]))
                    self.load(argparse.Namespace(name=ns.name))
                    if not self.config.data["plugins"][plugin["name"]]["enabled"]:
                        self.config.data["plugins"][plugin["name"]]["enabled"] = True
                        self.config.write_config()

    @cmd2.as_subcommand_to("plugins", "disable", disable_parser)
    @cmd2.with_category("Plugins commands")
    def disable(self, ns: argparse.Namespace):
        """Disable at startup a plugin by it name and unload it"""
        if (
            hasattr(self, "config")
            and self.config
            and hasattr(self, "shell")
            and self.shell
        ):
            for plugin in self.plugins:
                if ns.name == plugin["name"]:
                    if self.shell.debug:
                        self.shell.pwarning(
                            "DISABLE PLUGIN: {0}".format(plugin["name"])
                        )
                    self.unload(argparse.Namespace(name=ns.name))
                    if self.config.data["plugins"][plugin["name"]]["enabled"]:
                        self.config.data["plugins"][plugin["name"]]["enabled"] = False
                        self.config.write_config()

    @cmd2.as_subcommand_to("plugins", "load", load_parser)
    @cmd2.with_category("Plugins commands")
    def load(self, ns: argparse.Namespace):
        """Load a plugin by it name"""
        if (
            hasattr(self, "plugins")
            and self.plugins
            and hasattr(self, "shell")
            and self.shell
        ):
            for plugin in self.plugins:
                if ns.name == plugin["name"]:
                    if self.shell.debug:
                        self.shell.pwarning("LOAD PLUGIN: {0}".format(plugin["name"]))
                    plugin["object"].shell = self.shell
                    plugin["object"].load()
                    self.loaded_plugins.append(
                        "{0} {1}".format(plugin["name"], plugin["version"])
                    )

    @cmd2.as_subcommand_to("plugins", "unload", unload_parser)
    @cmd2.with_category("Plugins commands")
    def unload(self, ns: argparse.Namespace):
        """Unload a plugin by it name"""
        if (
            hasattr(self, "plugins")
            and self.plugins
            and hasattr(self, "shell")
            and self.shell
        ):
            for plugin in self.plugins:
                if ns.name == plugin["name"]:
                    if self.shell.debug:
                        self.shell.pwarning("UNLOAD PLUGIN: {0}".format(plugin["name"]))
                    plugin["object"].shell = self.shell
                    plugin["object"].unload()
                    try:
                        self.loaded_plugins.remove(
                            "{0} {1}".format(plugin["name"], plugin["version"])
                        )
                    except ValueError:
                        pass

    @cmd2.as_subcommand_to("plugins", "reload", reload_parser)
    @cmd2.with_category("Plugins commands")
    def reload(self, ns: argparse.Namespace):
        """Reload a plugin by it name"""
        if (
            hasattr(self, "plugins")
            and self.plugins
            and hasattr(self, "shell")
            and self.shell
        ):
            for plugin in self.plugins:
                if ns.name == plugin["name"]:
                    if self.shell.debug:
                        self.shell.pwarning("RELOAD PLUGIN: {0}".format(plugin["name"]))
                    plugin["object"].shell = self.shell
                    plugin["object"].reload()

    @cmd2.with_argparser(plugins_parser)
    @cmd2.with_category("Plugins")
    def do_plugins(self, ns: argparse.Namespace):
        handler = ns.cmd2_handler.get()  # pragma: no cover
        if handler is not None:  # pragma: no cover
            # Call whatever subcommand function was selected
            handler(ns)
        else:  # pragma: no cover
            # No subcommand was provided, so call help
            if hasattr(self, "shell") and self.shell:
                self.shell.perror("This command does nothing without sub-commands")
                self.shell.do_help("plugins")



import os
import cmd2
import argparse

from GLXShell.plugins.builtins import PLUGIN_VERSION
from GLXShell.plugins.builtins import PLUGIN_DESCRIPTION
from GLXShell.plugins.builtins import PLUGIN_WARRANTY

pwd_parser = argparse.ArgumentParser(
    description="The pwd utility shall write to standard output an absolute pathname of the current working "
    "directory, which does not contain the filenames dot or dot-dot. "
)
pwd_parser.add_argument(
    "-L",
    "--logical",
    action="store_true",
    help="If the PWD environment variable contains an absolute pathname of the current directory that does not "
    "contain the filenames dot or dot-dot, pwd shall write this pathname to standard output. Otherwise, "
    "the -L option shall behave as the -P option.",
)
pwd_parser.add_argument(
    "-P",
    "--physical",
    action="store_true",
    help="The absolute pathname written shall not contain filenames that, in the context of the pathname, "
    "refer to files of type symbolic link. ",
)
pwd_parser.add_argument(
    "--version", action="store_true", help="output version information and exit"
)


class GLXPwd(cmd2.CommandSet):
    def __init__(self):
        super().__init__()

    @staticmethod
    def pwd_print_version():
        cmd2.Cmd().poutput(
            "pwd ({name}) v{version}\n{licence}".format(
                name=PLUGIN_DESCRIPTION,
                version=PLUGIN_VERSION,
                licence=PLUGIN_WARRANTY,
            )
        )

    @staticmethod
    def pwd_print_logical():
        cmd2.Cmd().poutput("{pwd}".format(pwd=os.path.normpath(os.getcwd())))

    @staticmethod
    def pwd_print_not_logical():
        cmd2.Cmd().poutput("{pwd}".format(pwd=os.path.realpath(os.getcwd())))

    def pwd(self, logical, physical):
        if logical:
            self.pwd_print_logical()
        else:
            self.pwd_print_not_logical()
        return

    @cmd2.with_argparser(pwd_parser)
    @cmd2.with_category("Builtins")
    def do_pwd(self, args):
        if args.version:  # pragma: no cover
            self.pwd_print_version()
            return

        self.pwd(logical=args.logical, physical=args.physical)  # pragma: no cover



import os
import argparse
import cmd2

from GLXShell.plugins.builtins import PLUGIN_VERSION
from GLXShell.plugins.builtins import PLUGIN_DESCRIPTION
from GLXShell.plugins.builtins import PLUGIN_WARRANTY
from GLXShell.plugins.builtins import PLUGIN_VERSION

arch_parser = argparse.ArgumentParser(description="Print machine architecture.")
#      --help               display this help and exit
#      --version            output version information and exit
arch_parser.add_argument(
    "--version", action="store_true", help="output version information and exit"
)


class GLXArch(cmd2.CommandSet):
    def __init__(self):
        super().__init__()

    @property
    def result(self):
        return f"{os.uname().machine}"

    @staticmethod
    def arch_print_version():
        cmd2.Cmd().poutput(
            "arch ({name}) v{version}\n{licence}".format(
                name=PLUGIN_DESCRIPTION,
                version=PLUGIN_VERSION,
                licence=PLUGIN_WARRANTY,
            )
        )

    def arch(self):
        cmd2.Cmd().last_result = self.result
        cmd2.Cmd().poutput(self.result)

    @cmd2.with_argparser(arch_parser)
    @cmd2.with_category("Builtins")
    def do_arch(self, args):
        if args.version:  # pragma: no cover
            self.arch_print_version()
            return

        self.arch()  # pragma: no cover



# http://pwet.fr/man/linux/commandes/posix/touch/
# https://www.maizure.org/projects/decoded-gnu-coreutils/df.html
# https://pubs.opengroup.org/onlinepubs/9699919799/utilities/df.html

import math
import os
from argparse import RawDescriptionHelpFormatter

import cmd2
from tabulate import tabulate

from GLXShell.plugins.builtins import PLUGIN_DESCRIPTION
from GLXShell.plugins.builtins import PLUGIN_VERSION
from GLXShell.plugins.builtins import PLUGIN_WARRANTY

df_parser = cmd2.Cmd2ArgumentParser(
    description="The df utility shall write the amount of available space and file slots for file systems on which "
                "the invoking user has appropriate read access. File systems shall be specified by the file operands; "
                "when none are specified, information shall be written for all file systems. The format of the "
                "default output from df is unspecified, but all space figures are reported in 512-byte units, "
                "unless the -k option is specified. This output shall contain at least the file system names, "
                "amount of available space on each of these file systems, and, if no options other than -t are "
                "specified, the number of free file slots, or inodes, available; when -t is specified, the output "
                "shall contain the total allocated space as well.",
    add_help=False,
    formatter_class=RawDescriptionHelpFormatter
)
#      --help               display this help and exit
#      --version            output version information and exit
df_parser.add_argument(
    "--version", action="store_true", help="output version information and exit"
)
df_parser.add_argument('--help',
                       action='store_true',
                       help="display this help and exit"
                       )
df_parser.add_argument(
    "-h",
    dest="human_readable",
    action="store_true",
    default=False,
    help="print sizes in powers of 1024 (e.g., 1023M)",
)
df_parser.add_argument(
    "-k",
    dest="kilo",
    action='store_const', const=1024,
    help="use 1024-byte units, instead of the default 512-byte units, when writing space figures.",
)
df_parser.add_argument(
    "-P",
    dest="portability",
    action="store_true",
    default=True,
    help="produce a POSIX output",
)
df_parser.add_argument(
    "-t",
    dest="total",
    action="store_true",
    help="include total allocated-space figures in the output. ",
)
df_parser.add_argument(
    "file",
    completer=cmd2.Cmd.path_complete,
    nargs="?",
    const=0,
    help="A pathname of a file within the hierarchy of the desired file system. If a file other than a FIFO, "
         "a regular file, a directory, or a special file representing the device containing the file system (for "
         "example, /dev/dsk/0s1) is specified, the results are unspecified. If the file operand names a "
         "file other than a special file containing a file system, df shall write the amount of free space in the "
         "file system containing the specified file operand. Otherwise, df shall write the amount of free space in "
         "that file system. ",
)


class GLXDf(cmd2.CommandSet):
    def __init__(self):
        super().__init__()
        # Internal Variables

    @staticmethod
    def df_print_help():
        df_parser.print_help()

    @staticmethod
    def df_print_version():
        cmd2.Cmd().poutput(
            "df ({name}) v{version}\n{licence}".format(
                name=PLUGIN_DESCRIPTION,
                version=PLUGIN_VERSION,
                licence=PLUGIN_WARRANTY,
            )
        )

    def df(self, file=None, block_size=None, total=None, human_readable=None):
        """Exit this application"""
        # Return True to stop the command loop
        if block_size is None:
            block_size = 512

        devices_list = []
        if file:
            if not os.path.exists(file):
                cmd2.Cmd().perror(f"df: {file}: No such file or directory")
                cmd2.cmd2.last_result = 1
                return True

            if not os.access(file, os.R_OK) or not os.access(self.df_find_mount_point(file),
                                                             os.R_OK):  # pragma: no cover
                cmd2.Cmd().perror(f"df: {file}: Permission denied")
                cmd2.cmd2.last_result = 1
                return True

            for line in self.df_get_devices():
                if self.df_find_mount_point(file) == line[1]:
                    devices_list.append(self.df_get_device_information(
                        file_system_name=line[0],
                        file_system_root=line[1],
                        block_size=block_size,
                    ))

        else:
            for line in self.df_get_devices():
                devices_list.append(self.df_get_device_information(
                    file_system_name=line[0],
                    file_system_root=line[1],
                    block_size=block_size,
                ))

        if devices_list:
            if total:
                total_space_free, total_space_used, total_total_space = self.df_get_totals(
                    devices_list
                )

                devices_list.append(
                    ["total",
                     total_total_space,
                     total_space_used,
                     total_space_free,
                     "%d%%" % int(math.ceil(100 * (float(total_total_space - total_space_free) / total_total_space))),
                     "-"]
                )

            block_size_text, tabular_data = self.df_get_info_to_print(
                block_size,
                devices_list,
                human_readable
            )

            self.df_print_final(block_size_text, tabular_data)

            cmd2.cmd2.last_result = 0
        else:
            cmd2.cmd2.last_result = 1
        return True

    def df_get_info_to_print(self, block_size, devices_list, human_readable):
        block_size_text = f"{block_size}-blocks"
        if human_readable:
            tabular_data = []
            block_size_text = "Size"
            for line in devices_list:
                if str(line[1]) != '-' and str(line[2]) != '-' and str(line[3]) != '-':
                    tabular_data.append([line[0],
                                         self.sizeof(int(line[1]) * block_size),
                                         self.sizeof(int(line[2]) * block_size),
                                         self.sizeof(int(line[3]) * block_size),
                                         line[4],
                                         line[5]]
                                        )
                else:
                    tabular_data.append(line)
        else:
            tabular_data = devices_list

        return block_size_text, tabular_data

    @staticmethod
    def df_get_totals(devices_list):
        total_total_space = 0
        total_space_used = 0
        total_space_free = 0
        for device in devices_list:
            try:
                total_total_space += int(device[1])
            except ValueError:
                pass
            try:
                total_space_used += int(device[2])
            except ValueError:
                pass
            try:
                total_space_free += int(device[3])
            except ValueError:
                pass
        return total_space_free, total_space_used, total_total_space

    @staticmethod
    def df_print_final(block_size_text, tabular_data):
        cmd2.Cmd().poutput(
            tabulate(
                tabular_data=tabular_data,
                headers=["Filesystem",
                         block_size_text,
                         "Used",
                         "Available",
                         "Capacity",
                         "Mounted on"],
                tablefmt="plain",
                colalign=("left", "right", "right", "right", "right", "left")
            )
        )

    @cmd2.with_argparser(df_parser)
    @cmd2.with_category("Builtins")
    def do_df(self, args):

        if args.version:  # pragma: no cover
            self.df_print_version()
            cmd2.cmd2.last_result = 0
            return

        if args.help:  # pragma: no cover
            self.df_print_help()
            cmd2.cmd2.last_result = 0
            return

        self.df(
            file=args.file,
            block_size=args.kilo,
            total=args.total,
            human_readable=args.human_readable,
        )  # pragma: no cover

    @staticmethod
    def df_find_mount_point(path):
        if not os.path.islink(path):
            path = os.path.abspath(path)
        elif os.path.islink(path) and os.path.lexists(os.readlink(path)):  # pragma: no cover
            path = os.path.realpath(path)
        while not os.path.ismount(path):
            path = os.path.dirname(path)
            if os.path.islink(path) and os.path.lexists(os.readlink(path)):  # pragma: no cover
                path = os.path.realpath(path)
        return path

    @staticmethod
    def df_get_device_information(
            file_system_name=None,
            file_system_root=None,
            block_size=None):
        try:
            statvfs = os.statvfs(file_system_root)
            space_free = statvfs.f_bavail * statvfs.f_frsize / block_size
            total_space = statvfs.f_blocks * statvfs.f_frsize / block_size
            space_used = total_space - space_free
            if total_space == 0:
                percentage_used = '-'
            else:
                percentage_used = "%d%%" % int(math.ceil(100 * (float(total_space - space_free) / total_space)))

            return ["%s" % file_system_name,
                    "%d" % total_space,
                    "%d" % space_used,
                    "%d" % space_free,
                    "%s" % percentage_used,
                    "%s" % file_system_root,
                    ]
        except PermissionError:  # pragma: no cover
            return ["%s" % file_system_name,
                    "%s" % '-',
                    "%s" % '-',
                    "%s" % '-',
                    "%s" % '-',
                    "%s" % file_system_root,
                    ]

    def df_get_devices(self, file=None):
        if file is None and os.path.exists('/etc/mtab'):
            file = '/etc/mtab'
        if file is None and os.path.exists('/proc/mounts'):  # pragma: no cover
            file = '/proc/mounts'
        if file is None:  # pragma: no cover
            raise SystemError("Impossible to locate /etc/mtab or /proc/mounts file")

        file_entries = []
        for line in self.get_file_content(file=file).splitlines():
            if len(line.split()) < 4:  # pragma: no cover
                continue
            file_entries.append(line.split())
        return file_entries

    @staticmethod
    def get_file_content(file=None):
        if not os.path.exists(file):
            raise FileExistsError(f"{file} do not exist")
        if not os.access(file, os.R_OK):  # pragma: no cover
            raise PermissionError(f"{file} can't be read")

        with open(file) as datafile:
            return datafile.read().strip()

    @staticmethod
    def sizeof(value=None):
        """
        Convert a num to a human readable thing it use metric prefix.

        :param value: a ``value`` to translate for a future display
        :type value: int or float
        :return: str
        :raise TypeError: when ``value`` argument is not a int or float
        """
        #     Metric prefixes in everyday use:
        #
        #     yotta	Y	1000000000000000000000000	10 power 24
        #     zetta	Z	1000000000000000000000	10 power 21
        #     exa	E	1000000000000000000	10 power 18
        #     peta	P	1000000000000000	10 power 15
        #     tera	T	1000000000000	10 power 12
        #     giga	G	1000000000	10 power 9
        #     mega	M	1000000	10 power 6
        #     kilo	k	1000	10 power 3
        #     (none)	(none)	1	10 power 0

        # Exit a soon of possible
        if type(value) != int and type(value) != float and type(value) != int:
            raise TypeError("'value' must be a int or float type")

        suffix = ["", "k", "M", "G", "T", "P", "E", "Z", "Y"]
        i = 0 if value < 1 else int(math.log(value, 1024)) + 1
        v = value / math.pow(1024, i)
        v, i = (v, i) if v > 0.5 else (v * 1024, (i - 1 if i else 0))

        return str(str(int(round(v, 0))) + suffix[i])



from GLXShell.plugins.builtins import PLUGIN_NAME as BUILTINS_NAME
from GLXShell.plugins.builtins import PLUGIN_VERSION as BUILTINS_VERSION
from GLXShell.plugins.builtins import GLXShPluginBuiltins

APPLICATION_AUTHORS = ["Tuuuux"]
APPLICATION_DESCRIPTION = ""
APPLICATION_LICENSE = "GNU General Public License v3 or later (GPLv3+)"
APPLICATION_NAME = "glxsh"
APPLICATION_VERSION = "0.2.3"
APPLICATION_WARRANTY = """Copyright (C) 2020-2021 Galaxie Shell Project.
License GPLv3+: GNU GPL version 3 or later <https://gnu.org/licenses/gpl.html>.
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.
"""
APPLICATION_PLUGINS = [
    {
        "name": BUILTINS_NAME,
        "version": BUILTINS_VERSION,
        "object": GLXShPluginBuiltins(),
    },
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# It script it publish under GNU GENERAL PUBLIC APPLICATION_LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Curses Team, all rights reserved
import os

from GLXShell.libs.xdg_base_directory import XDGBaseDirectory


class GLXShPropertyHistory(XDGBaseDirectory):
    def __init__(self):
        XDGBaseDirectory.__init__(self)
        self.__persistent_history_file = None
        self.__persistent_history_length = None

        self.persistent_history_file = None
        self.persistent_history_length = None

    @property
    def persistent_history_file(self):
        return self.__persistent_history_file

    @persistent_history_file.setter
    def persistent_history_file(self, value=None):
        if value is None or value == "":
            value = os.path.join(self.config_path, "history")
        if value != self.persistent_history_file:
            self.__persistent_history_file = value

    @property
    def persistent_history_length(self):
        return self.__persistent_history_length

    @persistent_history_length.setter
    def persistent_history_length(self, value=None):
        if value is None:
            value = 500
        if type(value) != int:
            raise TypeError(
                "'persistent_history_length' property value must be int type or None"
            )
        if not value >= 0:
            raise ValueError(
                "'persistent_history_length' property value must be positive int"
            )
        if self.persistent_history_length != value:
            self.__persistent_history_length = value



import cmd2

from GLXShell.libs.properties.commands import GLXShellPropertyCommands


class GLXShellCommands(GLXShellPropertyCommands):
    """
    It class have responsibility to receive heritage from ``GLXShPropertyShell`` and ``GLXShellPropertyCommands``.

    Then the class provide ``GLXShellCommands.load()``, ``GLXShellCommands.unload()`` and ``GLXShellCommands.reload()``
    method's.

    It class is include by every plugins, for add method's use directly by the Plugins Manager.
    """

    def __init__(self):
        GLXShellPropertyCommands.__init__(self)

    def load(self):
        """
        It function provide load plugin capability to class it receive heritage from it class.

        ``shell`` property is require and should not be to None, that is done automatically by ``GLXSHell``
        """
        if (
            hasattr(self, "commands")
            and self.commands
            and hasattr(self, "shell")
            and self.shell
        ):
            for command in self.commands:
                try:
                    self.shell.register_command_set(command["object"])
                    if self.shell.debug:
                        self.shell.pwarning("LOAD COMMAND: {0}".format(command["name"]))
                except cmd2.CommandSetRegistrationError:
                    if self.shell.debug:
                        self.shell.perror(
                            "ALREADY LOADED COMMAND: {0}  ".format(command["name"])
                        )

    def unload(self):
        """
        It function provide unload plugin capability to class it receive heritage from it class.

        ``shell`` property is require and should not be to None, that is done automatically by ``GLXSHell``
        """
        if (
            hasattr(self, "commands")
            and self.commands
            and hasattr(self, "shell")
            and self.shell
        ):
            for command in self.commands:
                self.shell.unregister_command_set(command["object"])
                if self.shell.debug:
                    self.shell.pwarning("UNLOAD COMMAND: {0}".format(command["name"]))

    def reload(self):
        """
        It function provide reload plugin capability to class it receive heritage from it class.

        ``shell`` property is require and should not be to None, that is done automatically by ``GLXSHell``
        """
        self.unload()
        self.load()



from GLXShell.plugins.builtins import GLXShPluginBuiltins
from GLXShell.plugins.builtins import PLUGIN_VERSION


class GLXShPropertyPlugins(object):
    def __init__(self):
        self.__plugins = None
        self.plugins = None

    @property
    def plugins(self):
        """
        ``plugins`` property store the plugins list and assure to set plugins value only if have changed

        each item should have it dict form: {"name": "Hello", "object": object}

        :return: the plugins property value
        :rtype: list or None
        """
        return self.__plugins

    @plugins.setter
    def plugins(self, value=None):
        """
        Set ``plugins`` property value

        :param value: a plugins list or None
        :type value: list or None
        :raise TypeError: when property value is not a list or None
        """
        if value is None:
            value = [
                {
                    "name": "builtins",
                    "version": PLUGIN_VERSION,
                    "object": GLXShPluginBuiltins(),
                }
            ]
        if type(value) != list:
            raise TypeError("'commands' property value must be a list or None")
        if self.plugins != value:
            self.__plugins = value



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# It script it publish under GNU GENERAL PUBLIC APPLICATION_LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Curses Team, all rights reserved

import os
import getpass
import socket
from cmd2 import (
    ansi,
)
from GLXShell.libs.config import GLXShConfig


class GLXShPrompt(object):
    def __init__(self, *args, **kwargs):
        self.config = kwargs.get("config", GLXShConfig())

        have_change = False

        if "prompt" not in self.config.data:
            self.config.data["prompt"] = {}
            have_change = True

        if "show" not in self.config.data["prompt"]:
            self.config.data["prompt"]["show"] = {}
            have_change = True

        if have_change:
            self.config.write_config()

        self.__prompt_show_info = None
        self.__prompt_show_cursor = None

        try:
            self.prompt_show_cursor = self.config.data["prompt"]["show"]["cursor"]
        except KeyError:
            self.prompt_show_cursor = None
        try:
            self.prompt_show_info = self.config.data["prompt"]["show"]["info"]
        except KeyError:
            self.prompt_show_info = None

    @property
    def prompt_show_info(self):
        """
        That property is use in front on the user space configuration file, and be like a storage value wrapper.

        If the value change , the configuration file will be rewrite with fresh information's.

        :return: If ``True`` the prompt contain information's line.
        :rtype: bool
        """
        return self.__prompt_show_info

    @prompt_show_info.setter
    def prompt_show_info(self, value=None):
        if value is None:
            value = False
        if type(value) != bool:
            raise TypeError(
                "'prompt_show_info' property value must be a bool type or None"
            )
        if self.prompt_show_info != value:
            self.__prompt_show_info = value
        if (
            "info" not in self.config.data["prompt"]["show"]
            or self.config.data["prompt"]["show"]["info"] != value
        ):
            self.config.data["prompt"]["show"]["info"] = value
            self.config.write_config()

    @property
    def prompt_show_cursor(self):
        """
        That property is use in front on the user space configuration file, and be like a storage value wrapper.

        If the value change , the configuration file will be rewrite with fresh information's.

        :return: If ``True`` the prompt display something before the cursor.
        :rtype: bool
        """
        return self.__prompt_show_cursor

    @prompt_show_cursor.setter
    def prompt_show_cursor(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError(
                "'prompt_show_cursor' property value must be a bool type or None"
            )
        if self.prompt_show_cursor != value:
            self.__prompt_show_cursor = value
        if (
            "cursor" not in self.config.data["prompt"]["show"]
            or self.config.data["prompt"]["show"]["cursor"] != value
        ):
            self.config.data["prompt"]["show"]["cursor"] = value
            self.config.write_config()

    def onchange_prompt_show_cursor(self, param_name, old_value, new_value):
        """Internal function call by ``cmd2`` python module during a ``set``"""
        pass

    def onchange_prompt_show_info(self, param_name, old_value, new_value):
        """Internal function call by ``cmd2`` python module during a ``set``"""
        pass

    @property
    def prompt_env_is_virtual(self):
        """
        Return ``True`` if a virtual environment is detected.

        :return: ``True`` if environments variables *VIRTUAL_ENV* or *CONDA_DEFAULT_ENV* are set
        :rtype: bool
        """
        if os.environ.get("VIRTUAL_ENV") or os.environ.get("CONDA_DEFAULT_ENV"):
            return True
        else:
            return False

    @property
    def prompt_env_name_text(self):
        """
        It property return the activated virtual environment surrounded by parentheses

        *VIRTUAL_ENV* and *CONDA_DEFAULT_ENV* environments variables are used during venv detection.

        **Example**

        .. code:: text

            (venv)

        :return: the basename of directory announced by var *VIRTUAL_ENV* or *CONDA_DEFAULT_ENV*
        :rtype: str
        """
        if os.environ.get("VIRTUAL_ENV") or os.environ.get("CONDA_DEFAULT_ENV"):
            env_path = os.environ.get("VIRTUAL_ENV", "")
            if len(env_path) == 0:
                env_path = os.environ.get("CONDA_DEFAULT_ENV", "")

            return "({env_name}) ".format(env_name=os.path.basename(env_path))
        else:
            return ""

    @property
    def prompt_username_text(self):
        """
        That property return the ``getpass.getuser()`` value. That is used by the prompt for display username.

        **Example**

        .. code:: text

            me

        :return: ``getpass.getuser()``
        :rtype: str
        """
        return getpass.getuser()

    @property
    def prompt_hostname_text(self):
        """
        That property return the ``socket.gethostname()`` value. That is used by the prompt for display hostname.

        **Example**

        .. code:: text

            localdomain

        :return: ``socket.gethostname()``
        :rtype: str
        """
        return socket.gethostname()

    @property
    def prompt_path_text(self):
        """
        That property return the current working directory. That is used by the prompt for display path.

        .. note:: the path is user expanded.

        **Example**

        .. code:: text

            ~/galaxie-shell/docs

        :return: current working directory
        :rtype: str
        """
        return os.getcwd().replace(os.path.realpath(os.path.expanduser("~")), "~")

    @property
    def prompt_symbol_text(self):
        """
        If UID = 0 return #

        If not return $

        :return: # if ``os.getuid() == 0`` else $
        :rtype: str
        """
        if os.getuid() == 0:  # pragma: no cover
            return "#"
        else:  # pragma: no cover
            return "$"

    @property
    def prompt_cursor_text(self):
        """
        That property return the character just before cursor.

        **Example**

        .. code:: text

            >

        :return: char just before the blinking cursor
        :rtype: str
        """
        return ">"

    @property
    def prompt_to_display(self):
        """
        The function is use by the GLXShell class for get prompt text to display.

        That is responsibility of GLXShellPrompt class to know what display where GLXShell class only get the text.

        **Example**

        .. code:: text

            (venv) me@localdomain:~/galaxie-shell/docs $
            >

        :return: a big string it correspond to the entire prompt to display
        :rtype: str
        """
        prompt = []
        if self.prompt_show_info:
            prompt.append(
                "{venv}{username}{arobase}{hostname}:{path}{symbol} ".format(
                    venv=self.prompt_env_name_text,
                    username=ansi.style(self.prompt_username_text, fg="green", bold=True),
                    arobase=ansi.style("@", fg="green", bold=True),
                    hostname=ansi.style(self.prompt_hostname_text, fg="green", bold=True),
                    path=ansi.style(self.prompt_path_text, fg="blue", bold=True),
                    symbol=self.prompt_symbol_text,
                )
            )
        if self.prompt_show_cursor:
            prompt.append("{cursor} ".format(cursor=self.prompt_cursor_text))

        return "\n".join(prompt)



import cmd2
import sys
import platform

from GLXShell.plugins.builtins import PLUGIN_VERSION
from GLXShell.plugins.builtins import PLUGIN_DESCRIPTION
from GLXShell.plugins.builtins import PLUGIN_WARRANTY

uname_parser = cmd2.Cmd2ArgumentParser()
uname_parser.add_argument(
    "-a",
    "--all",
    action="store_true",
    help="print all information, in the following order, except omit -p and -i if unknown",
)
uname_parser.add_argument(
    "-s", "--kernel-name", action="store_true", help="print the kernel name"
)
uname_parser.add_argument(
    "-n", "--nodename", action="store_true", help="print the network node hostname"
)
uname_parser.add_argument(
    "-r", "--kernel-release", action="store_true", help="print the kernel release"
)
uname_parser.add_argument(
    "-v", "--kernel-version", action="store_true", help="print the kernel version"
)
uname_parser.add_argument(
    "-m", "--machine", action="store_true", help="print the machine hardware name"
)
uname_parser.add_argument(
    "-p",
    "--processor",
    action="store_true",
    help="print the processor type (non-portable)",
)
uname_parser.add_argument(
    "-i",
    "--hardware-platform",
    action="store_true",
    help="print the hardware platform (non-portable)",
)
uname_parser.add_argument(
    "-o", "--operating-system", action="store_true", help="print the operating system"
)
uname_parser.add_argument(
    "--version", action="store_true", help="output version information and exit"
)


class GLXUname(cmd2.CommandSet):
    def __init__(self, *args, **kwargs):
        super().__init__()
        self.stdout = kwargs.get("stdout", sys.stdout)
        self.stdin = kwargs.get("stdin", sys.stdin)
        self.stderr = kwargs.get("stdin", sys.stderr)

    def uname_print_version(self):
        cmd2.Cmd().poutput(
            "uname ({name}) v{version}\n{licence}".format(
                name=PLUGIN_DESCRIPTION,
                version=PLUGIN_VERSION,
                licence=PLUGIN_WARRANTY,
            )
        )
        self.stdout.flush()

    @staticmethod
    def uname(
        all_info=False,
        kernel_name=False,
        nodename=False,
        kernel_release=False,
        kernel_version=False,
        machine=False,
        processor=False,
        hardware_platform=False,
        operating_system=False,
    ):
        line = []
        if all_info:
            kernel_name = True
            nodename = True
            kernel_release = True
            kernel_version = True
            machine = True
            processor = False
            hardware_platform = False
            operating_system = True

        if kernel_name:
            line.append(platform.uname().system)

        if nodename:
            line.append(platform.uname().node)

        if kernel_release:
            line.append(platform.uname().release)

        if kernel_version:
            line.append(platform.uname().version)

        if machine:
            line.append(platform.uname().machine)

        if processor:
            if len(platform.uname().processor) <= 0:
                line.append("unknown")
            else:  # pragma: no cover
                line.append(platform.uname().processor)

        if hardware_platform:
            line.append("unknown")

        if operating_system:
            line.append(sys.platform)

        # print the default value
        if len(line) <= 0:
            line.append(platform.uname().system)

        if line:
            cmd2.Cmd().poutput(f'{" ".join(line)}')
            cmd2.Cmd().last_result = f'{" ".join(line)}'

    @cmd2.with_argparser(uname_parser)
    @cmd2.with_category("Builtins")
    def do_uname(self, args):
        """
        Print certain system information.

        :param args: Arguments like -a
        :type args: args or None
        """
        if args.version:  # pragma: no cover
            self.uname_print_version()
            return

        self.uname(
            all_info=args.all,
            kernel_name=args.kernel_name,
            nodename=args.nodename,
            kernel_release=args.kernel_release,
            kernel_version=args.kernel_version,
            machine=args.machine,
            processor=args.processor,
            hardware_platform=args.hardware_platform,
            operating_system=args.operating_system,
        )  # pragma: no cover



from GLXShell.libs.config import GLXShConfig


class GLXShPropertyConfig(object):
    def __init__(self):
        self.__config = None
        self.config = None

    @property
    def config(self):
        """
        ``config`` property store the config object and assure to set config value only if have changed

        :return: a Galaxie Shell instance or None
        :rtype: GLXShell.Shell or None
        """
        return self.__config

    @config.setter
    def config(self, value=None):
        """
        Set ``config`` property value

        :param value: a Galaxie Shell Config instance or None
        :type value: GLXShConfig or None
        :raise TypeError: when property value is not a Galaxie Shell Config instance or None
        """
        if value is None:
            value = GLXShConfig()
        if value is not None and not isinstance(value, GLXShConfig):
            raise TypeError(
                "'shell' property value must be a Galaxie Shell Config instance or None"
            )
        if self.config != value:
            self.__config = value



import os
import cmd2


from GLXShell.plugins.builtins import PLUGIN_VERSION
from GLXShell.plugins.builtins import PLUGIN_DESCRIPTION
from GLXShell.plugins.builtins import PLUGIN_WARRANTY

# Inspired from: http://pwet.fr/man/linux/commandes/posix/cd/


cd_parser = cmd2.Cmd2ArgumentParser(
    prog="CD",
    description="Change the working directory",
)

cd_parser.add_argument(
    "directory",
    nargs="?",
    const=0,
    help="An absolute or relative pathname of the directory that shall become the new working directory.",
)
cd_parser.add_argument(
    "-P",
    "--physical",
    action="store_true",
    default=False,
    help="Symbolic link components shall be resolved before dot-dot components are processed",
)
cd_parser.add_argument(
    "-L",
    "--logical",
    action="store_true",
    default=False,
    help="Symbolic link components shall not be resolved before dot-dot components are processed",
)
cd_parser.add_argument(
    "--version", action="store_true", help="output version information and exit"
)


class GLXCd(cmd2.CommandSet):
    def __init__(self):
        super().__init__()
        self.__curpath = None
        self.__directory = None
        self.__logical = None
        self.__physical = None
        self.__can_continue = None

        self.curpath = None
        self.directory = None
        self.logical = None
        self.physical = None
        self.can_continue = None

    @property
    def curpath(self):
        """
        ``curpath`` represents an intermediate value used to simplify the description of the algorithm used by cd.

        :return: value used to simplify the description
        :rtype: str or None
        """
        return self.__curpath

    @curpath.setter
    def curpath(self, value=None):
        """
        Set the ``curpath`` property value

        :param value: value used to simplify the description
        :type value: str or None
        :raise TypeError: when ``directory`` property value is not str type or None
        """
        if value is not None and type(value) != str:
            raise TypeError("'curpath' property value must be str type or None")
        if self.curpath != value:
            self.__curpath = value

    @property
    def directory(self):
        """
        An absolute or relative pathname of the directory that shall become the new working directory.

        :return: directory that shall become the new working directory
        :rtype: str or None
        """
        return self.__directory

    @directory.setter
    def directory(self, value=None):
        """
        Set the ``directory`` property value

        :param value: directory that shall become the new working directory
        :type value: str or None
        :raise TypeError: when ``directory`` property value is not str type or None
        """
        if value is not None and type(value) != str:
            raise TypeError("'directory' property value must be str type or None")
        if self.directory != value:
            self.__directory = value

    @property
    def logical(self):
        """
        The ``logical`` property is use as control during the 9 steps of the posix sequence

        Default: False

        :return: True if handle the operand dot-dot logically
        :rtype: bool
        """
        return self.__logical

    @logical.setter
    def logical(self, value=None):
        """
        Set the ``logical`` property value

        Default: False

        :param value: True if handle the operand dot-dot logically
        :type value: bool or None
        """
        if value is None:
            value = False
        if type(value) != bool:
            raise TypeError("'logical' property value must be bool type or None")
        if self.logical != value:
            self.__logical = value

    @property
    def physical(self):
        """
        The ``physical`` property is use as control during the 9 steps of the posix sequence

        Default: False

        :return: True if handle the operand dot-dot physically
        :rtype: bool
        """
        return self.__physical

    @physical.setter
    def physical(self, value=None):
        """
        Set the ``physical`` property value

        Default: True

        :param value: True if handle the operand dot-dot physically
        :type value: bool or None
        """
        if value is None:
            value = False
        if type(value) != bool:
            raise TypeError("'physical' property value must be bool type or None")
        if self.physical != value:
            self.__physical = value

    @property
    def can_continue(self):
        """
        The ``working`` property is use as control during the 9 steps of the posix sequence

        Default: True

        :return: True if any step can be operate
        :rtype: bool
        """
        return self.__can_continue

    @can_continue.setter
    def can_continue(self, value=None):
        """
        Set the ``working`` property value

        Default: True

        :param value: True if any step can be operate
        :type value: bool or None
        """
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError("'working' property value must be bool type or None")
        if self.can_continue != value:
            self.__can_continue = value

    @staticmethod
    def cd_print_version():
        cmd2.Cmd().poutput(
            "cd ({name}) v{version}\n{licence}".format(
                name=PLUGIN_DESCRIPTION,
                version=PLUGIN_VERSION,
                licence=PLUGIN_WARRANTY,
            )
        )

    def cd(self, directory=None, logical=None, physical=None):
        # Init
        self.can_continue = True
        self.curpath = None

        self.directory = directory
        self.logical = logical
        self.physical = physical

        # Logical
        if self.logical and self.physical:
            self.physical = False
        if not self.logical and not self.physical:
            self.logical = True

        if self.directory == "-":
            if os.environ.get("OLDPWD"):
                self.directory = os.environ.get("OLDPWD")
            else:
                self.directory = os.environ.get("PWD")

        elif self.directory is not None:
            self.directory = os.path.expanduser(self.directory)

        # Start the posix sequence
        self._step_1()
        self._step_2()
        self._step_3()
        self._step_4()
        self._step_5()
        self._step_6()
        self._step_7()
        self._step_8()
        self._step_9()

        # Inform cmd2 directly
        cmd2.Cmd().last_result = os.getcwd()
        cmd2.Cmd().set_window_title(
            os.path.normpath(os.getcwd()).replace(
                os.path.realpath(os.path.expanduser("~")), "~"
            )
        )

    def _step_1(self):
        """
        If no directory operand is given and the HOME environment variable is empty or undefined,
        the default behavior is implementation-defined and no further steps shall be taken.
        """
        if self.can_continue and self.directory is None and not os.environ.get("HOME"):
            self.can_continue = False

    def _step_2(self):
        """
        If no directory operand is given and the HOME environment variable is set to a non-empty value,
        the *cd* utility shall behave as if the directory named in the HOME environment variable was
        specified as the directory operand.
        """
        if self.can_continue and self.directory is None and os.environ.get("HOME"):
            self.directory = os.environ.get("HOME")

    def _step_3(self):
        """
        If the directory operand begins with a slash character, set ``curpath`` to the operand and proceed to step 7.
        """
        if self.can_continue and self.directory.startswith(os.path.sep):
            self.curpath = self.directory
            self._step_7()

    def _step_4(self):
        """
        If the first component of the directory operand is dot or dot-dot, proceed to step 6.
        """
        if (
            self.can_continue
            and self.directory.startswith("..")
            or self.directory.startswith(".")
        ):
            self._step_6()

    def _step_5(self):
        """
        Starting with the first pathname in the colon-separated pathnames of CDPATH
        (see the ENVIRONMENT VARIABLES section) if the pathname is non-null,
        test if the concatenation of that pathname, a slash character, and the directory operand names a directory.

        If the pathname is null, test if the concatenation of dot, a slash character, and the operand names a directory.

        In either case, if the resulting string names an existing directory,
        set curpath to that string and proceed to step 7.

        Otherwise, repeat this step with the next pathname in CDPATH until all pathnames have been tested.
        """
        # 5. Starting with the first pathname in the colon-separated pathnames of CDPATH
        # (see the ENVIRONMENT VARIABLES section)
        if self.can_continue and os.environ.get("CDPATH"):
            for pathname in os.environ.get("CDPATH").split(":"):
                if pathname:
                    # if the pathname is non-null, test if the concatenation of that pathname,
                    # a slash character, and the directory operand names a directory.
                    if os.path.isdir(os.path.join(pathname, self.directory)):
                        self.curpath = os.path.join(pathname, self.directory)
                        self._step_7()
                        break
                    # If the pathname is null, test if the concatenation of dot, a slash character, and
                    # the operand names a directory.
                    # set curpath to that string and proceed to step 7.
                    elif os.path.isdir(
                        os.path.join(".{0}".format(os.path.sep), self.directory)
                    ):
                        self.curpath = os.path.join(
                            ".{0}".format(os.path.sep), self.directory
                        )
                        self._step_7()
                        break
                    # elif os.path.isdir(pathname):
                    #     # In either case, if the resulting string names an existing directory,
                    #     self.curpath = pathname
                    #     self._step_7()
                    #     break

                # Otherwise, repeat this step with the next pathname in CDPATH until all pathnames have been tested.

    def _step_6(self):
        """
        Set ``curpath`` to the string formed by the concatenation of the value of *PWD* , a slash character,
        and the operand.
        """
        if self.can_continue:
            self.curpath = os.path.join(os.environ.get("PWD"), self.directory)

    def _step_7(self):
        """
        If the **-P** option is in effect, the cd utility shall perform actions equivalent to the chdir() function,
         called with ``curpath`` as the path argument.

        If these actions succeed, the *PWD* environment variable shall be set to an absolute pathname for
        the current working directory and shall not contain filename components that,
        in the context of pathname resolution, refer to a file of type symbolic link.

        If there is insufficient permission on the new directory, or on any parent of that directory,
        to determine the current working directory, the value of the *PWD* environment variable is unspecified.

        If the actions equivalent to ``chdir()`` fail for any reason, the cd utility shall display
        an appropriate error message and not alter the PWD environment variable.

        Whether the actions equivalent to ``chdir()`` succeed or fail, no further steps shall be taken.
        """
        if self.can_continue and self.physical:
            # Make sure the directory exists, is a directory, and we have read access
            err = None

            if not os.path.isdir(self.curpath):
                err = f"cd: {self.curpath}: No such file or directory"
            elif not os.access(self.curpath, os.R_OK):  # pragma: no cover
                err = f"cd: {self.curpath}: Permission denied"
            else:
                try:
                    os.chdir(self.curpath)
                except Exception as ex:  # pragma: no cover
                    err = f"{ex}"
                else:
                    if os.environ.get("OLDPWD") != os.environ["PWD"]:
                        os.environ["OLDPWD"] = os.environ["PWD"]
                    if os.environ.get("PWD") != os.path.normpath(os.getcwd()):
                        os.environ["PWD"] = os.path.normpath(os.getcwd())

            if err:
                cmd2.Cmd().perror(err)

            self.can_continue = False

    def _step_8(self):
        """
        The ``curpath`` value shall then be converted to canonical form as follows, considering each component
        from beginning to end, in sequence:

        a. Dot components and any slashes that separate them from the next component shall be deleted.

        b. For each dot-dot component, if there is a preceding component and it is neither root nor dot-dot,
        the preceding component, all slashes separating the preceding component from dot-dot, dot-dot and all slashes
        separating dot-dot from the following component shall be deleted.

        c. An implementation may further simplify ``curpath`` by removing any trailing slash characters that are not
        also leading slashes, replacing multiple non-leading consecutive slashes with a single slash, and replacing
        three or more leading slashes with a single slash. If, as a result of this canonicalization, the ``curpath``
        variable is null, no further steps shall be taken.
        """
        if self.can_continue:
            self.curpath = os.path.abspath(self.curpath)

    def _step_9(self):
        """
        The cd utility shall then perform actions equivalent to the ``chdir()`` function called with ``curpath`` as
        the path argument.

        If these actions failed for any reason, the cd utility shall display an appropriate error message
        and no further steps shall be taken.

        The *PWD* environment variable shall be set to ``curpath``.
        """
        if self.can_continue:
            err = None

            if not os.path.isdir(self.curpath):
                err = f"cd: {self.curpath}: No such file or directory"
            elif not os.access(self.curpath, os.R_OK):  # pragma: no cover
                err = f"cd: {self.curpath}: Permission denied"
            else:
                try:
                    os.chdir(self.curpath)
                except Exception as ex:  # pragma: no cover
                    err = f"{ex}"
                else:
                    if os.environ.get("OLDPWD") != os.environ["PWD"]:
                        os.environ["OLDPWD"] = os.environ["PWD"]
                    if os.environ["PWD"] != self.curpath:
                        os.environ["PWD"] = self.curpath

            if err:
                cmd2.Cmd().perror(err)
            self.can_continue = False

    @cmd2.with_argparser(cd_parser)
    @cmd2.with_category("Builtins")
    def do_cd(self, args):
        if args.version:  # pragma: no cover
            self.cd_print_version()
            return

        self.cd(
            directory=args.directory, logical=args.logical, physical=args.physical
        )  # pragma: no cover

    @staticmethod
    def complete_cd(text, line, begidx, endidx):
        return cmd2.Cmd().path_complete(
            text=text,
            line=line,
            begidx=begidx,
            endidx=endidx,
            path_filter=os.path.isdir,
        )



import os
import cmd2
import argparse

from GLXShell.plugins.builtins import PLUGIN_VERSION
from GLXShell.plugins.builtins import PLUGIN_DESCRIPTION
from GLXShell.plugins.builtins import PLUGIN_WARRANTY

dir_parser = argparse.ArgumentParser()
dir_parser.add_argument(
    "-l",
    "--long",
    action="store_true",
    help="display in long format with one item per line",
)
dir_parser.add_argument(
    "--version", action="store_true", help="output version information and exit"
)


class GLXDir(cmd2.CommandSet):
    def __init__(self):
        super().__init__()

    @staticmethod
    def dir_print_version():
        cmd2.Cmd().poutput(
            "dir ({name}) v{version}\n{licence}".format(
                name=PLUGIN_DESCRIPTION,
                version=PLUGIN_VERSION,
                licence=PLUGIN_WARRANTY,
            )
        )

    @staticmethod
    def dir():
        # Get the contents as a list
        contents = os.listdir(os.getcwd())
        for f in contents:
            cmd2.Cmd().poutput(f"{f}")

        cmd2.Cmd().last_result = contents

    @cmd2.with_argparser(dir_parser, with_unknown_args=True)
    @cmd2.with_category("Builtins")
    def do_dir(self, args, unknown):
        """List contents of current directory."""
        if args.version:  # pragma: no cover
            self.dir_print_version()
            return

        # No arguments for this commands
        if unknown:  # pragma: no cover
            cmd2.Cmd().perror("dir does not take any positional arguments:")
            dir_parser.print_help()
            cmd2.Cmd().last_result = "Bad arguments"
            return

        self.dir()  # pragma: no cover



import cmd2


class GLXShSettable(object):
    def __init__(self, *args, **kwargs):
        self.shell = kwargs.get("shell", args[0])
        # del cmd2.Cmd.do_py
        # del cmd2.Cmd.do_edit
        # del cmd2.Cmd.do_shortcuts
        #del cmd2.Cmd.do_pyscript
        #del cmd2.Cmd.do_set
        #del cmd2.Cmd.do_alias
        #del cmd2.Cmd.do_load
        # del cmd2.Cmd.do_run_pyscript
        # del cmd2.Cmd.do_run_script


    def load_settable(self):
        """
        Set every settable for the shell object
        """

        self.shell.add_settable(
            cmd2.Settable(
                "intro_show_exec",
                bool,
                "Show the exec information line",
                onchange_cb=self.shell.onchange_intro_show_exec,
                settable_object=self,
            )
        )
        self.shell.add_settable(
            cmd2.Settable(
                "intro_show_holotape",
                bool,
                "Show the holotape information line",
                onchange_cb=self.shell.onchange_intro_show_holotape,
                settable_object=self,
            )
        )
        self.shell.add_settable(
            cmd2.Settable(
                "intro_show_license",
                bool,
                "Show the license information line",
                onchange_cb=self.shell.onchange_intro_show_license,
                settable_object=self,
            )
        )
        self.shell.add_settable(
            cmd2.Settable(
                "intro_show_loader",
                bool,
                "Show the loader information line",
                onchange_cb=self.shell.onchange_intro_show_loader,
                settable_object=self,
            )
        )
        self.shell.add_settable(
            cmd2.Settable(
                "intro_show_memory_free",
                bool,
                "Show the memory_free information line",
                onchange_cb=self.shell.onchange_intro_show_memory_free,
                settable_object=self,
            )
        )
        self.shell.add_settable(
            cmd2.Settable(
                "intro_show_memory_total",
                bool,
                "Show the memory_total information line",
                onchange_cb=self.shell.onchange_intro_show_memory_total,
                settable_object=self,
            )
        )

        self.shell.add_settable(
            cmd2.Settable(
                "intro_show_rom",
                bool,
                "Show the rom information line",
                onchange_cb=self.shell.onchange_intro_show_rom,
                settable_object=self,
            )
        )
        self.shell.add_settable(
            cmd2.Settable(
                "intro_show_spacing",
                bool,
                "Show the spacing information line",
                onchange_cb=self.shell.onchange_intro_show_spacing,
                settable_object=self,
            )
        )
        self.shell.add_settable(
            cmd2.Settable(
                "intro_show_title",
                bool,
                "Show the first line into with version information",
                onchange_cb=self.shell.onchange_intro_show_title,
                settable_object=self,
            )
        )
        # PROMPT
        # self.set_window_title(self.window_title)
        self.shell.shortcuts.update({"ll": "shell ls -lah --color=auto"})
        self.shell.add_settable(
            cmd2.Settable(
                "prompt_show_info",
                bool,
                "Show prompt information line",
                onchange_cb=self.shell.onchange_prompt_show_info,
                settable_object=self.shell,
            )
        )
        self.shell.add_settable(
            cmd2.Settable(
                "prompt_show_cursor",
                bool,
                "Show cursor line",
                onchange_cb=self.shell.onchange_prompt_show_cursor,
                settable_object=self.shell,
            )
        )



class GLXShellPropertyCommands(object):
    def __init__(self):
        self.__commands = None
        self.commands = None

    @property
    def commands(self):
        """
        ``commands`` property store the commands list and assure to set commands value only if have changed

        each item should have it dict form: {"name": "Hello", "object": object}

        :return: the commands property value
        :rtype: list or None
        """
        return self.__commands

    @commands.setter
    def commands(self, value=None):
        """
        Set ``commands`` property value

        :param value: a commands list or None
        :type value: list or None
        :raise TypeError: when property value is not a list or None
        """
        if value is None:
            value = []
        if type(value) != list:
            raise TypeError("'commands' property value must be a list or None")
        if self.commands != value:
            self.__commands = value



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# It script it publish under GNU GENERAL PUBLIC APPLICATION_LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Shell Team, all rights reserved

import os
import json

from GLXShell.libs.xdg_base_directory import XDGBaseDirectory


class Singleton(type):
    def __init__(cls, name, bases, dictionary):
        super(Singleton, cls).__init__(name, bases, dictionary)
        cls.instance = None

    def __call__(cls, *args, **kw):
        if cls.instance is None:
            cls.instance = super(Singleton, cls).__call__(*args)
        return cls.instance


class GLXShConfig(XDGBaseDirectory, metaclass=Singleton):
    """
    Config provide method to work with Galaxie Shell configuration file.

    The configuration file respect XDG standard, then the configuration file is store on standardized path.
    (it can be change but by default that is ``~/.config/glxsh/config.json``)

    GLXShConfig have two settings views, one store on a configuration file , and one store on ``data`` property.

    ``data`` property is the property use by each class for get the setting, that is responsibility to each
    class's to verify if those setting have change then call ``GLXShConfig.write_config()`` in case.

    .. note:: Config is a Straightforward implementation of the Singleton Pattern.

    Configuration file example: ~/.config/glxsh/config.json

    .. code:: json

        {
            "intro": {
                "show": {
                    "exec": true,
                    "holotape": false,
                    "license": false,
                    "loader": false,
                    "memory_free": true,
                    "memory_total": true,
                    "rom": true,
                    "spacing": false,
                    "title": true
                }
            },
            "plugins": {
                "builtins": {
                    "enabled": true
                }
            },
            "prompt": {
                "show": {
                    "cursor": true,
                    "info": true
                }
            }
        }
    """

    def __init__(self):
        XDGBaseDirectory.__init__(self)

        self.__file = None
        self.__data = None

        self.file = None
        self.data = None

        self.touch_config()
        self.load_config()

    @property
    def file(self):
        """
        Property it store configuration file path

        Note: Default is XDGBaseDirectory/glxsh

        :return: the config file path
        :rtype: str
        """
        return self.__file

    @file.setter
    def file(self, value=None):
        """
        Set the ``file`` property value

        Note: if ``file`` property value is set to ``None``
        it restore default value provide by XDGBaseDirectory join by APPLICATION_NAME value fount in __init__.py on the
        root of the project.

        :param value: the config file path
        :type value: str or None
        """
        if value is None:
            value = os.path.join(self.config_path, "config.json")
        if type(value) != str:
            raise TypeError("'config_file' property value must be a str type or None")
        if value != self.file:
            self.__file = value

    @property
    def data(self):
        """
        A dictionary it contain all the setting. That property is for get or store data setting.

        .. note:: If ``data`` property value is ``None``, it restore a empty dictionary setting, but not write it.

        :return: the setting
        :rtype: dict
        """
        return self.__data

    @data.setter
    def data(self, value=None):
        """
        Set the ``data`` property

        :param value: the setting
        :return: dict or None
        :raise TypeError: When ``data`` property value is not dict type or None
        """
        if value is None:
            value = {}
        if type(value) != dict:
            raise TypeError("'data' property value must be a dict type or None")
        if self.data != value:
            self.__data = value

    def touch_config(self):
        """
        Create directories where store the configuration file and if the config file do not exist
        create one with a empty dictionary.

        The configuration file path is get by ``file`` property
        """
        if not os.path.exists(os.path.dirname(self.file)):
            os.makedirs(os.path.dirname(self.file))
        if not os.path.exists(self.file):
            with open(self.file, "w") as outfile:
                json.dump({}, outfile)

    def load_config(self):
        """
        Load the configuration file and update ``data`` property, the configuration file path is get by
        ``file`` property
        """
        with open(self.file, "r") as json_data_file:
            self.data.update(json.load(json_data_file))

    def write_config(self):
        """
        Write ``data`` property value content inside the configuration file path get by
        ``file`` property
        """
        with open(self.file, "w") as outfile:
            json.dump(self.data, outfile, sort_keys=True, indent=4)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# It script it publish under GNU GENERAL PUBLIC APPLICATION_LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Shell Team, all rights reserved

# Inspired by:
import os
import stat
import io
import random

from GLXShell import APPLICATION_NAME


def new_id():
    """
    Generate a ID like 'E59E8457', two chars by two chars it's a random HEX

    **Default size:**
        8
    **Default chars:**
        'ABCDEF0123456789'
    **Benchmark**
       +----------------+---------------+----------------------------------------------+
       | **Iteration**  | **Duration**  | **CPU Information**                          |
       +----------------+---------------+----------------------------------------------+
       | 10000000       | 99.114s       | Intel(R) Core(TM) i7-2860QM CPU @ 2.50GHz    |
       +----------------+---------------+----------------------------------------------+
       | 1000000        | 9.920s        | Intel(R) Core(TM) i7-2860QM CPU @ 2.50GHz    |
       +----------------+---------------+----------------------------------------------+
       | 100000         | 0.998s        | Intel(R) Core(TM) i7-2860QM CPU @ 2.50GHz    |
       +----------------+---------------+----------------------------------------------+
       | 10000          | 0.108s        | Intel(R) Core(TM) i7-2860QM CPU @ 2.50GHz    |
       +----------------+---------------+----------------------------------------------+

    :return: a string it represent a unique ID
    :rtype: str
    """
    return "%02x%02x%02x%02x".upper() % (
        random.randint(0, 255),
        random.randint(0, 255),
        random.randint(0, 255),
        random.randint(0, 255),
    )


def get_os_temporary_dir():
    """
    Get the OS default dir , the better as it can.

    It suppose to be cross platform

    :return: A tmp dir path
    :rtype: str
    """
    text_flags = os.O_RDWR | os.O_CREAT | os.O_EXCL
    if hasattr(os, "O_NOFOLLOW"):
        text_flags |= os.O_NOFOLLOW

    bin_flags = text_flags
    if hasattr(os, "O_BINARY"):  # pragma: no cover
        bin_flags |= os.O_BINARY

    directory_list = list()

    # First, try the environment.
    for envname in "TMPDIR", "TEMP", "TMP":
        dirname = os.getenv(envname)
        if dirname:
            directory_list.append(os.path.abspath(dirname))

    directory_list.extend(["/tmp", "/var/tmp", "/usr/tmp"])

    for directory in directory_list:
        if directory != os.path.curdir:
            directory = os.path.abspath(directory)

        filename = os.path.join(
            directory, "{0}-{1}{2}".format(APPLICATION_NAME, new_id(), ".cp")
        )
        try:
            fd = os.open(filename, bin_flags, 0o600)
            try:
                try:
                    with io.open(fd, "wb", closefd=False) as fp:
                        fp.write(b"Test")
                finally:
                    os.close(fd)
            finally:
                os.unlink(filename)
            return directory

        except PermissionError:  # pragma: no cover
            break  # no point trying more names in this directory
        except OSError:  # pragma: no cover
            break  # no point trying more names in this directory
    raise FileNotFoundError(  # pragma: no cover
        "No usable temporary directory found in %s" % directory_list
    )


def control_directory(directory=None, mode=0o700):
    """
    Internal function it create a directory if not exist

    :param directory:
    :type: str
    :param mode: the permission mode of the directory if created example:``0o700``
    :type mode: int
    :return: the directory path or None
    :rtype: str
    :raise TypeError: when ``directory`` is not a str type
    :raise TypeError: when ``directory`` is not a int type
    """
    if type(directory) != str:
        raise TypeError("directory must be a str type")

    if type(mode) != int:
        raise TypeError("mode must be a int type")

    if not os.path.isdir(directory):
        os.makedirs(directory, mode=mode)

    return directory


class XDGBaseDirectory(object):
    def __init__(self):
        self.__resource = None

        self.set_resource(APPLICATION_NAME)

    @property
    def xdg_data_home(self):
        """
        $XDG_DATA_HOME defines the base directory relative to which user specific data files should be stored.

        If $XDG_DATA_HOME is either not set or empty, a default equal to $HOME/.local/share should be used.

        :return: base directory relative to which user specific data files
        :rtype: str
        """
        if os.environ.get("XDG_DATA_HOME") and os.path.isabs(
            os.environ.get("XDG_DATA_HOME")
        ):
            return os.environ.get("XDG_DATA_HOME")

        return control_directory(
            os.path.join(os.path.expanduser("~"), ".local", "share")
        )

    @property
    def xdg_config_home(self):
        """
        $XDG_CONFIG_HOME defines the base directory relative to which user specific configuration files
        should be stored.

        If $XDG_CONFIG_HOME is either not set or empty, a default equal to $HOME/.config should be used.

        :return: base directory relative to which user specific configuration files
        :rtype: str
        """
        if os.environ.get("XDG_CONFIG_HOME") and os.path.isabs(
            os.environ.get("XDG_CONFIG_HOME")
        ):
            return os.environ.get("XDG_CONFIG_HOME")

        return control_directory(os.path.join(os.path.expanduser("~"), ".config"))

    @property
    def xdg_data_dirs(self):
        """
        $XDG_DATA_DIRS defines the preference-ordered set of base directories to search for data files in addition to
        the $XDG_DATA_HOME base directory.

        The directories in $XDG_DATA_DIRS should be separated with a colon ':'.

        If $XDG_DATA_DIRS is either not set or empty, a value equal to /usr/local/share/:/usr/share/ should be used.

        :return: preference-ordered set of base directories to search for data files separated with a colon ':'
        :rtype: str
        """
        if os.environ.get("XDG_DATA_DIRS"):
            tmp_list = [self.xdg_data_home]
            for path in os.environ.get("XDG_DATA_DIRS").split(":"):
                if os.path.isabs(path):
                    tmp_list.append(control_directory(path))

            return ":".join(tmp_list)

        return "/usr/local/share/:/usr/share/"

    @property
    def xdg_config_dirs(self):
        """
        $XDG_CONFIG_DIRS defines the preference-ordered set of base directories to search for configuration files
        in addition to the $XDG_CONFIG_HOME base directory.

        The directories in $XDG_CONFIG_DIRS should be separated with a colon ':'.

        If $XDG_CONFIG_DIRS is either not set or empty, a value equal to /etc/xdg should be used.

        :return: preference-ordered set of base directories to search for configuration files separated with a colon ':'
        :rtype: str
        """
        if os.environ.get("XDG_CONFIG_DIRS"):
            tmp_list = [self.xdg_config_home]
            for path in os.environ.get("XDG_CONFIG_DIRS").split(":"):
                if os.path.isabs(path) and path not in tmp_list:
                    tmp_list.append(control_directory(path))

            return ":".join(tmp_list)

        return ":".join([self.xdg_config_home, "/etc/xdg"])

    @property
    def xdg_cache_home(self):
        """
        $XDG_CACHE_HOME defines the base directory relative to which user specific non-essential data files
        should be stored.

        If $XDG_CACHE_HOME is either not set or empty, a default equal to $HOME/.cache should be used.

        :return: base directory relative to which user specific non-essential data files
        :rtype: str
        """
        if os.environ.get("XDG_CACHE_HOME") and os.path.isabs(
            os.environ.get("XDG_CACHE_HOME")
        ):
            return os.environ.get("XDG_CACHE_HOME")
        return os.path.join(os.path.expanduser("~"), ".cache")

    @property
    def xdg_runtime_dir(self):
        """
        $XDG_RUNTIME_DIR defines the base directory relative to which user-specific non-essential runtime files and
        other file objects (such as sockets, named pipes, ...) should be stored.

        If $XDG_RUNTIME_DIR is not set applications should fall back to a replacement directory with similar
        capabilities and print a warning message.

        Applications should use this directory for communication and synchronization purposes and should not
        place larger files in it, since it might reside in runtime memory and cannot necessarily be swapped out to disk.

        :return: base directory relative to which user-specific non-essential runtime files
        :rtype: str
        """
        if os.environ.get("XDG_RUNTIME_DIR") and os.path.isabs(
            os.environ.get("XDG_RUNTIME_DIR")
        ):
            return os.environ["XDG_RUNTIME_DIR"]

        fallback = os.path.join(
            get_os_temporary_dir(),
            "glxcurses-runtime-dir-fallback-{0}".format(os.geteuid()),
        )

        if os.path.isfile(fallback):
            os.remove(fallback)

        if os.path.islink(fallback):
            os.unlink(fallback)

        if os.path.isdir(fallback):
            if os.stat(fallback).st_uid != os.geteuid() or (
                os.stat(fallback).st_mode & (stat.S_IRWXG | stat.S_IRWXO)
            ):
                os.rmdir(fallback)

        return control_directory(fallback)

    @property
    def resource(self):
        """
        ``resource`` should normally be the name of your application or a shared resource.

        :return: name of your application or a shared resource
        :rtype: str
        """
        return self.__resource

    @resource.setter
    def resource(self, value):
        """
        Set the ``resource`` property value.

        You should not use it setter directly, then you should take a look to ``set_resource`` function

        :param value: name of your application or a shared resource
        :type value: str
        :raise TypeError: When ``resource`` value is not a str type
        """
        if type(value) != str:
            raise TypeError("'resource' value must be a str")
        if self.resource != value:
            self.__resource = value

    def set_resource(self, *resource):
        """
        Set the ``resource`` property value.

        :param resource: should normally be the name of your application
        :type resource: tuple like 'Hello', '42'
        :raise AssertionError: If when join the tuple start by os.path.sep typically ``/``
        """
        resource = os.path.join(*resource)
        if resource.startswith(os.path.sep):
            raise AssertionError(
                "'resource' value can not start by {0}".format(os.path.sep)
            )
        self.resource = resource

    @property
    def config_path(self):
        """
        Ensure ``$XDG_CONFIG_HOME/<resource>/`` exists, and return its path.

        :return: ``$XDG_CONFIG_HOME/<resource>/`` path
        :rtype: str
        """
        return control_directory(os.path.join(self.xdg_config_home, self.resource))

    @property
    def config_paths(self):
        """
        Returns an iterator which gives each directory named 'resource' in the configuration search path.

        Information provided by earlier directories should take precedence over later ones, and the user-specific
        config dir comes first.

        :return: pre-ordered set of base directories to search for configuration files directory for ``resource``
        :rtype: list
        """
        tmp_list = [self.config_path]
        for config_dir in self.xdg_config_dirs.split(":"):
            if os.path.exists(os.path.join(config_dir, self.resource)):
                if os.path.join(config_dir, self.resource) not in tmp_list:
                    tmp_list.append(os.path.join(config_dir, self.resource))
        return tmp_list

    @property
    def data_path(self):
        """
        Ensure ``$XDG_DATA_HOME/<resource>/`` exists, and return its path.

        :return: ``$XDG_DATA_HOME/<resource>/`` path
        :rtype: str
        """
        return control_directory(os.path.join(self.xdg_data_home, self.resource))

    @property
    def data_paths(self):
        """
        Returns an iterator which gives each directory named 'resource' in the application data search path.

        Information provided by earlier directories should take precedence over later ones.

        :return: preference-ordered set of base directories to search for data files directory for ``resource``
        :rtype: list
        """
        tmp_list = [self.data_path]
        for data_dir in self.xdg_data_dirs.split(":"):
            if os.path.exists(os.path.join(data_dir, self.resource)):
                if os.path.join(data_dir, self.resource) not in tmp_list:
                    tmp_list.append(os.path.join(data_dir, self.resource))
        return tmp_list

    @property
    def cache_path(self):
        """
        Ensure ``$XDG_CACHE_HOME/<resource>/`` exists, and return its path.

        :return: ``$XDG_CACHE_HOME/<resource>/`` path
        :rtype: str
        """
        return control_directory(os.path.join(self.xdg_cache_home, self.resource))



import os
import argparse
import cmd2
import time

from GLXShell.plugins.builtins import PLUGIN_DESCRIPTION
from GLXShell.plugins.builtins import PLUGIN_WARRANTY
from GLXShell.plugins.builtins import PLUGIN_VERSION

sleep_parser = argparse.ArgumentParser(
    description="Pause for NUMBER seconds.  SUFFIX may be 's' for seconds (the default), 'm' for minutes, "
                "'h' for hours or 'd' for days.  Unlike most implementations that require NUMBER be an integer, "
                "here NUMBER may be an arbitrary floating point number.  Given two or more arguments, pause for the "
                "amount of time specified by the sum of their values.")
#      --help               display this help and exit
#      --version            output version information and exit
sleep_parser.add_argument(
    "--version", action="store_true", help="output version information and exit"
)
sleep_parser.add_argument(
    "NUMBER",
    nargs=argparse.ZERO_OR_MORE,
    help="[SUFFIX] delay for a specified amount of time",
)


class GLXSleep(cmd2.CommandSet):
    def __init__(self):
        super().__init__()
        # Internal Variables
        self.__work_with = None

        # First Init
        self.work_with = None
        self.seconds_per_minute = 60
        self.seconds_per_hour = 3600
        self.seconds_per_day = 86400

    @property
    def work_with(self):
        return self.__work_with

    @work_with.setter
    def work_with(self, value=None):
        if value is None:
            value = "seconds"

        if type(value) != str:
            raise TypeError("'work_with' property value must be a str type or None")

        if str(value).lower() not in ['seconds', 'minutes', 'hours', 'days']:
            raise ValueError("'work_with' property value allowed value: 'seconds', 'minutes', 'hours', 'days'")

        if self.work_with != value:
            self.__work_with = value

    @property
    def result(self):
        return f"{os.uname().machine}"

    @staticmethod
    def sleep_print_version():
        cmd2.Cmd().poutput(
            "sleep ({name}) v{version}\n{licence}".format(
                name=PLUGIN_DESCRIPTION,
                version=PLUGIN_VERSION,
                licence=PLUGIN_WARRANTY,
            )
        )

    def sleep(self, values=None):
        time.sleep(self.values_to_seconds(values=values))
        return True

    @cmd2.with_argparser(sleep_parser)
    @cmd2.with_category("Builtins")
    def do_sleep(self, args):
        # print(args.NUMBER)
        if args.version:  # pragma: no cover
            self.sleep_print_version()
            return

        if len(args.NUMBER) > 0:  # pragma: no cover
            self.sleep(values=args.NUMBER)
        else:
            sleep_parser.print_usage()  # pragma: no cover

    def values_to_seconds(self, values=None):
        if values is None:
            values = ["0"]

        if type(values) != list:
            raise TypeError("'values' arguments must be a list or None")

        seconds = 0.0
        for value in values:
            if value[-1:].lower() == "d":
                print(str(value)[:-1])
                seconds += self.days_to_seconds(days=str(value)[:-1])
            elif value[-1:].lower() == "m":
                seconds += self.minutes_to_seconds(minutes=str(value)[:-1])
            elif value[-1:].lower() == "h":
                seconds += self.hours_to_seconds(hours=str(value)[:-1])
            elif value[-1:].lower() == "s":
                seconds += self.seconds_to_seconds(seconds=str(value)[:-1])
            else:
                seconds += self.seconds_to_seconds(seconds=value)

        return seconds

    @staticmethod
    def days_to_seconds(days=None):
        if days is None:
            days = 0
        try:
            float(days)
        except ValueError:
            raise TypeError("'days' arguments must be a int , float type or None")
        return float(86400.0 * float(days))

    @staticmethod
    def hours_to_seconds(hours=None):
        if hours is None:
            hours = 0
        try:
            float(hours)
        except ValueError:
            raise TypeError("'hours' arguments must be a int , float type or None")
        return float(3600.0 * float(hours))

    @staticmethod
    def minutes_to_seconds(minutes=None):
        if minutes is None:
            minutes = 0
        try:
            float(minutes)
        except ValueError:
            raise TypeError("'minutes' arguments must be a int , float type or None")
        return float(60.0 * float(minutes))

    @staticmethod
    def seconds_to_seconds(seconds=None):
        if seconds is None:
            seconds = 0
        try:
            return float(seconds)
        except ValueError:
            raise TypeError("'seconds' arguments must be a int , float type or None")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# It script it publish under GNU GENERAL PUBLIC APPLICATION_LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Shell Team, all rights reserved

import os
from setuptools import setup
import codecs


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), "r") as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith("APPLICATION_VERSION"):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


pre_version = get_version("GLXShell/__init__.py")

if os.environ.get("CI_COMMIT_TAG"):
    version = os.environ["CI_COMMIT_TAG"]
else:
    if os.environ.get("CI_JOB_ID"):
        version = os.environ["CI_JOB_ID"]
    else:
        version = pre_version

with open("README.rst") as f:
    long_description = f.read()

setup(
    name="galaxie-shell",
    version=version,
    description="Galaxie Shell is a autonomous shell ready for CI and low tech OS",
    long_description=long_description,
    long_description_content_type="text/x-rst; charset=UTF-8",
    url="https://gitlab.com/Tuuux/galaxie-shell",
    project_urls={
        'Read the Docs': 'https://galaxie-shell.readthedocs.io/',
        'GitLab': 'https://gitlab.com/Tuuux/galaxie-shell',
    },
    author="Tuuux",
    author_email="tuxa@rtnp.org",
    license="GNU General Public License v3 or later (GPLv3+)",
    packages=["GLXShell",
              "GLXShell.libs",
              "GLXShell.libs.properties",
              "GLXShell.plugins",
              "GLXShell.plugins.builtins",
              ],
    entry_points={"console_scripts": ["glxsh= GLXShell.glxsh:main"]},
    zip_safe=False,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    setup_requires=["green", "wheel", "pyinstaller"],
    tests_require=[
        "wheel",
        "cmd2",
        "tabulate",
    ],
    install_requires=[
        "wheel",
        "cmd2",
        "tabulate",
    ],
)



import os
import cmd2
import argparse

from GLXShell.plugins.builtins import PLUGIN_VERSION
from GLXShell.plugins.builtins import PLUGIN_DESCRIPTION
from GLXShell.plugins.builtins import PLUGIN_WARRANTY

mkdir_parser = argparse.ArgumentParser(
    description="Create the DIRECTORY(ies), if they do not already exist."
)
mkdir_parser.add_argument(
    "DIRECTORY",
    nargs=argparse.ZERO_OR_MORE,
    help="DIRECTORY(ies), to create",
    completer=cmd2.Cmd.path_complete,
)
mkdir_parser.add_argument(
    "-m",
    "--mode",
    nargs="?",
    const=1,
    type=str,
    default="755",
    help="set file mode (as in chmod), not a=rwx - umask",
)
mkdir_parser.add_argument(
    "-p",
    "--parents",
    action="store_true",
    default=False,
    help="no error if existing, make parent directories as needed",
)
mkdir_parser.add_argument(
    "-v",
    "--verbose",
    action="store_true",
    default=False,
    help="print a message for each created directory",
)
#
# mkdir_parser.add_argument(
#     "-Z",
#     action="store_true",
#     default=False,
#     help="set SELinux security context of each created directory to the default type"
# )
# mkdir_parser.add_argument(
#     "--context",
#     action="store_true",
#     type=str,
#     default="",
#     help="like -Z, or if CTX is specified then set the SELinux or SMACK security context to CTX"
# )
mkdir_parser.add_argument(
    "--version", action="store_true", help="output version information and exit"
)


class GLXMkdir(cmd2.CommandSet):
    def __init__(self):
        super().__init__()

    @staticmethod
    def mkdir_print_version():
        cmd2.Cmd().poutput(
            "mkdir ({name}) v{version}\n{licence}".format(
                name=PLUGIN_DESCRIPTION,
                version=PLUGIN_VERSION,
                licence=PLUGIN_WARRANTY,
            )
        )

    @staticmethod
    def mkdir_print_created_directory(directory, verbose):
        if verbose:
            cmd2.Cmd().poutput("mkdir: created directory '{0}'".format(directory))

    @staticmethod
    def mkdir_print_cannot_create_directory(directory):
        cmd2.Cmd().perror(
            "mkdir: cannot create directory '{0}': File exists".format(directory)
        )
        return

    def mkdir_with_no_parents(self, directory, mode, verbose):
        if mode:
            mode = int(mode, 8)
        if os.path.exists(directory):
            self.mkdir_print_cannot_create_directory(directory=directory)
        else:
            self.mkdir_print_created_directory(directory=directory, verbose=verbose)
            os.mkdir(path=directory, mode=mode)

    def mkdir_with_parents(self, directory, mode, verbose):
        if mode:
            mode = int(mode, 8)
        directory_to_create = ""
        for sub_directory in directory.split(os.path.sep):
            directory_to_create = os.path.join(directory_to_create, sub_directory)
            if not os.path.isdir(directory):
                self.mkdir_print_created_directory(
                    directory=directory_to_create, verbose=verbose
                )
                os.mkdir(path=directory_to_create, mode=mode)
            else:
                self.mkdir_print_cannot_create_directory(directory=directory_to_create)

    def mkdir(self, directories, parents, mode, verbose):
        for directory in directories:
            if parents:
                self.mkdir_with_parents(directory=directory, mode=mode, verbose=verbose)
            else:
                self.mkdir_with_no_parents(
                    directory=directory, mode=mode, verbose=verbose
                )

    @cmd2.with_argparser(mkdir_parser)
    @cmd2.with_category("Builtins")
    def do_mkdir(self, args):
        if args.version:  # pragma: no cover
            self.mkdir_print_version()
            return

        if not args.DIRECTORY:  # pragma: no cover
            mkdir_parser.print_help()
            return

        self.mkdir(
            directories=args.DIRECTORY,
            parents=args.parents,
            mode=args.mode,
            verbose=args.verbose,
        )  # pragma: no cover





import os
import cmd2
import argparse

from GLXShell.plugins.builtins import PLUGIN_VERSION
from GLXShell.plugins.builtins import PLUGIN_DESCRIPTION
from GLXShell.plugins.builtins import PLUGIN_WARRANTY

rmdir_parser = argparse.ArgumentParser(
    prog="RMDIR", description="Remove the DIRECTORY(ies), if they are empty."
)
rmdir_parser.add_argument(
    "DIRECTORY",
    nargs=argparse.ZERO_OR_MORE,
    help="DIRECTORY(ies), to delete",
    completer=cmd2.Cmd.path_complete
)

rmdir_parser.add_argument(
    "--ignore-fail-on-non-empty",
    action="store_true",
    default=False,
    help="ignore each failure that is solely because a directory is non-empty",
)

rmdir_parser.add_argument(
    "-p",
    "--parents",
    action="store_true",
    default=False,
    help="remove DIRECTORY and its ancestors; e.g., 'rmdir -p a/b/c' is similar to 'rmdir a/b/c a/b a'",
)
rmdir_parser.add_argument(
    "-v",
    "--verbose",
    action="store_true",
    default=False,
    help="output a diagnostic for every directory processed",
)
rmdir_parser.add_argument(
    "--version", action="store_true", help="output version information and exit"
)


class GLXRmDir(cmd2.CommandSet):
    exit_code: int

    @staticmethod
    def rmdir_print_version():
        cmd2.Cmd().poutput(
            "rmdir ({name}) v{version}\n{licence}".format(
                name=PLUGIN_DESCRIPTION,
                version=PLUGIN_VERSION,
                licence=PLUGIN_WARRANTY,
            )
        )

    @staticmethod
    def rmdir_print_removing_directory(directory, verbose):
        if verbose:
            cmd2.Cmd().poutput("rmdir: removing directory '{0}'".format(directory))

    @staticmethod
    def rmdir_print_directory_not_empty(directory, ignore_fail_on_non_empty):
        if not ignore_fail_on_non_empty:
            cmd2.Cmd().perror(
                "rmdir: failed to remove '{0}': Directory not empty".format(directory)
            )
            return

    @staticmethod
    def rmdir_print_no_such_file_or_directory(directory):
        cmd2.Cmd().perror(
            "rmdir: failed to remove '{0}': No such file or directory".format(directory)
        )
        return

    def rmdir_with_parent(self, directory, verbose, ignore_fail_on_non_empty):
        # Create a list item for each directories
        directory_to_remove = ""
        directory_to_remove_list = []
        for sub_directory in directory.split(os.path.sep):
            directory_to_remove = os.path.join(directory_to_remove, sub_directory)
            directory_to_remove_list.append(directory_to_remove)

        # Work with the reversed directories list
        for reversed_dir in reversed(directory_to_remove_list):
            if os.path.exists(reversed_dir) and os.path.isdir(reversed_dir):
                if not os.listdir(reversed_dir):
                    self.rmdir_print_removing_directory(
                        directory=reversed_dir, verbose=verbose
                    )
                    os.rmdir(path=reversed_dir)
                else:
                    self.rmdir_print_directory_not_empty(
                        directory=reversed_dir,
                        ignore_fail_on_non_empty=ignore_fail_on_non_empty,
                    )
            else:
                self.rmdir_print_no_such_file_or_directory(directory=reversed_dir)

    def rmdir_with_not_parent(self, directory, verbose, ignore_fail_on_non_empty):
        if os.path.exists(directory) and os.path.isdir(directory):
            if not os.listdir(directory):
                self.rmdir_print_removing_directory(
                    directory=directory, verbose=verbose
                )
                os.rmdir(path=directory)
            else:
                self.rmdir_print_directory_not_empty(
                    directory=directory,
                    ignore_fail_on_non_empty=ignore_fail_on_non_empty,
                )
        else:
            self.rmdir_print_no_such_file_or_directory(directory=directory)

    def rmdir(self, directories, parents, verbose, ignore_fail_on_non_empty):
        for directory in directories:
            if parents:
                self.rmdir_with_parent(
                    directory=directory,
                    verbose=verbose,
                    ignore_fail_on_non_empty=ignore_fail_on_non_empty,
                )
            else:
                self.rmdir_with_not_parent(
                    directory=directory,
                    verbose=verbose,
                    ignore_fail_on_non_empty=ignore_fail_on_non_empty,
                )

    @cmd2.with_argparser(rmdir_parser)
    @cmd2.with_category("Builtins")
    def do_rmdir(self, args):
        if args.version:  # pragma: no cover
            self.rmdir_print_version()
            return

        if not args.DIRECTORY:  # pragma: no cover
            rmdir_parser.print_help()
            return

        self.rmdir(
            directories=args.DIRECTORY,
            parents=args.parents,
            verbose=args.verbose,
            ignore_fail_on_non_empty=args.ignore_fail_on_non_empty,
        )  # pragma: no cover



# !/usr/bin/env python
# -*- coding: utf-8 -*-

# It script it publish under GNU GENERAL PUBLIC APPLICATION_LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Shell Team, all rights reserved

import os
import sys
import platform
from shutil import get_terminal_size

from GLXShell.libs.config import GLXShConfig


def get_memory_total():
    try:
        return os.sysconf("SC_PAGE_SIZE") * os.sysconf("SC_PHYS_PAGES")
    except ValueError:  # pragma: no cover
        return 0


def get_memory_available():
    try:
        return os.sysconf("SC_PAGE_SIZE") * os.sysconf("SC_AVPHYS_PAGES")
    except ValueError:  # pragma: no cover
        return 0


def get_size(size, suffix="B"):
    """
    Scale bytes to its proper format

    **Example:**
        1253656 => '1.20MB'
        1253656678 => '1.17GB'

    :param size: bytes size to convert
    :type size: int
    :param suffix: what suffix is add at the end of the returned value
    :type: str
    """
    for unit in ["", "K", "M", "G", "T", "P"]:
        if size < 1024:
            return f"{size:.2f}{unit}{suffix}"
        size /= 1024


class GLXShIntro(object):
    """
    It Class have responsibility to provide text to display with system information's.

    The class contain many property for user configuration, for let him capability to control it class setting from the
    ``glxsh`` , configuration file or from anywhere on the code.

    The entry point is the function ``intro_to_display()``

    **config**: ``GLXShConfig``
        A instance of GLXShConfig or None for create a new one.
        Generally you haven't to set it, that is use by tests for not destroy user space config during tests.
    **rows**: ``int``
        The terminal rows number, actually not use
    **columns**: ``int``
        The terminal columns

    """

    def __init__(self, *args, **kwargs):
        self.__intro_show_title = None
        self.__intro_show_spacing = None
        self.__intro_show_license = None
        self.__intro_show_loader = None
        self.__intro_show_exec = None
        self.__intro_show_memory_total = None
        self.__intro_show_memory_free = None
        self.__intro_show_holotape = None
        self.__intro_show_rom = None

        self.config = kwargs.get("config", GLXShConfig())

        have_change = False
        if "intro" not in self.config.data:
            self.config.data["intro"] = {}
            have_change = True

        if "show" not in self.config.data["intro"]:
            self.config.data["intro"]["show"] = {}
            have_change = True

        if have_change:
            self.config.write_config()

        try:
            self.intro_show_exec = self.config.data["intro"]["show"]["exec"]
        except KeyError:
            self.intro_show_exec = None
        try:
            self.intro_show_holotape = self.config.data["intro"]["show"]["holotape"]
        except KeyError:
            self.intro_show_holotape = None
        try:
            self.intro_show_license = self.config.data["intro"]["show"]["license"]
        except KeyError:
            self.intro_show_license = None
        try:
            self.intro_show_loader = self.config.data["intro"]["show"]["loader"]
        except KeyError:
            self.intro_show_loader = None
        try:
            self.intro_show_memory_free = self.config.data["intro"]["show"][
                "memory_free"
            ]
        except KeyError:
            self.intro_show_memory_free = None
        try:
            self.intro_show_memory_total = self.config.data["intro"]["show"][
                "memory_total"
            ]
        except KeyError:
            self.intro_show_memory_total = None
        try:
            self.intro_show_rom = self.config.data["intro"]["show"]["rom"]
        except KeyError:
            self.intro_show_rom = None
        try:
            self.intro_show_spacing = self.config.data["intro"]["show"]["spacing"]
        except KeyError:
            self.intro_show_spacing = None
        try:
            self.intro_show_title = self.config.data["intro"]["show"]["title"]
        except KeyError:
            self.intro_show_title = None

    @property
    def intro_line_exec(self):
        """
        That property return a formatted line it contain a upper string with python env and version information's.

        They information's are search on environment vars *VIRTUAL_ENV* or *CONDA_DEFAULT_ENV* and module ``sys``
        python module.

        Example:

        .. code:: text

            EXEC VENV PYTHON 3.7.3

        :return: line formatted with python env and version information's
        :rtype: str
        """
        if os.environ.get("VIRTUAL_ENV") or os.environ.get("CONDA_DEFAULT_ENV"):
            env_path = os.environ.get("VIRTUAL_ENV", "")
            if len(env_path) == 0:
                env_path = os.environ.get("CONDA_DEFAULT_ENV", "")
            exec_venv = "({env_name}) ".format(
                env_name=os.path.basename(env_path)
            ).upper()
        else:
            exec_venv = ""
        return "EXEC {exec_venv}PYTHON {python_version_major}.{python_version_minor}.{python_version_micro}".format(
            exec_venv=exec_venv,
            python_version_major=sys.version_info.major,
            python_version_minor=sys.version_info.minor,
            python_version_micro=sys.version_info.micro,
        )

    @property
    def intro_line_holotape(self):
        """
        That property return a formatted line it contain a upper string with style text.

        It will normally be use for plugins information's

        Example:

        .. code:: text

            NO HOLOTAPE FOUND

        :return: line formatted with style
        :rtype: str
        """
        return "NO HOLOTAPE FOUND"

    @property
    def intro_line_license(self):
        """
        That property return a formatted line it contain a upper string with style text.

        It will normally be use for plugins information's

        Example:

        .. code:: text

            GNU GENERAL PUBLIC APPLICATION_LICENSE GPL-3.0

        :return: line formatted with style
        :rtype: str
        """
        _license = ""

        if hasattr(self, "shell") and self.shell:
            if hasattr(self.shell, "license"):
                _license = self.shell.license

        return _license.upper()

    @property
    def intro_line_loader(self):
        """
        That property return a formatted line it contain a system information text.

        Information's are found on ``platform`` python module.

        Example:

        .. code:: text

            LOADER #1 SMP DEBIAN 4.19.146-1 (2020-09-17)

        :return: line formatted with style
        :rtype: str
        """
        return "LOADER {0}".format(platform.version().upper())

    @property
    def intro_line_memory_free(self):
        """
        That property return a formatted line it contain a system memory free information's text.

        Example:

        .. code:: text

            21.60GB FREE

        :return: memory free information's
        :rtype: str
        """
        return "{mem_available} FREE".format(
            mem_available=get_size(get_memory_available())
        )

    @property
    def intro_line_memory_total(self):
        """
        That property return a formatted line it contain a system memory total information's text.

        Example:

        .. code:: text

            31.36GB RAM SYSTEM

        :return: memory total information's
        :rtype: str
        """
        return "{mem_total} RAM SYSTEM".format(mem_total=get_size(get_memory_total()))

    @property
    def intro_line_rom(self):
        """
        That property return a formatted line it contain a upper string with style text.

        It return plugins information's

        Example:

        .. code:: text

            LOAD APPLICATION_PLUGINS(1): DEITRIX 303

        :return: plugins information's
        :rtype: str
        """
        value = 0
        value_2 = ""
        if hasattr(self, "shell") and self.shell:
            if hasattr(self.shell, "loaded_plugins"):
                value = len(self.shell.loaded_plugins)
                value_2 = ", ".join(self.shell.loaded_plugins)

        return "LOAD PLUGINS({0}): {1}".format(value, value_2.upper())

    @property
    def intro_line_title(self):
        """
        That property return a string formatted as a line it contain a upper string with application and version
        information's.

        They information's are search in ``__init__.py`` file on the root of the project.

        .. code:: python

            APPLICATION_NAME= "SuperShell"
            APPLICATION_VERSION= "42.1"

        The ``columns`` property value is use to calculate the width and title position.

        Example:

        .. code:: text

            ************************** SUPERSHELL V42.1 ****************************

        :return: line formatted with ``title`` property information's
        :rtype: str
        """
        app_name = ""
        version = ""
        if hasattr(self, "shell") and self.shell:
            if hasattr(self.shell, "name"):
                app_name = self.shell.name
            if hasattr(self.shell, "version"):
                version = self.shell.version

        title = "{app_name} V{version}".format(
            app_name=app_name.upper(), version=version.upper()
        )
        columns, rows = get_terminal_size()
        return "{text_inner} {text} {text_outer}".format(
            text=title,
            text_inner="*" * int((int(columns) / 2) - (len(title) / 2) - 1),
            text_outer="*" * int((int(columns) / 2) - (len(title) / 2) - 1),
        )

    @property
    def intro_show_exec(self):
        """
        That property is use in front on the user space configuration file, and be like a storage value wrapper.

        If the value change , the configuration file will be rewrite with fresh information's.

        :return: If ``True`` the EXEC line is allow to be display.
        :rtype: bool
        """
        return self.__intro_show_exec

    @intro_show_exec.setter
    def intro_show_exec(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError("'exec' property value must be a bool type or None")
        if self.intro_show_exec != value:
            self.__intro_show_exec = value
        if (
            "exec" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["exec"] != value
        ):
            self.config.data["intro"]["show"]["exec"] = value
            self.config.write_config()

    @property
    def intro_show_holotape(self):
        """
        That property is use in front on the user space configuration file, and be like a storage value wrapper.

        If the value change , the configuration file will be rewrite with fresh information's.

        :return: If ``True`` the EXEC line is allow to be display.
        :rtype: bool
        """
        return self.__intro_show_holotape

    @intro_show_holotape.setter
    def intro_show_holotape(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError("'holotape' property value must be a bool type or None")
        if self.intro_show_holotape != value:
            self.__intro_show_holotape = value
        if (
            "holotape" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["holotape"] != value
        ):
            self.config.data["intro"]["show"]["holotape"] = value
            self.config.write_config()

    @property
    def intro_show_license(self):
        """
        That property is use in front on the user space configuration file, and be like a storage value wrapper.

        If the value change , the configuration file will be rewrite with fresh information's.

        :return: If ``True`` the APPLICATION_LICENSE line is allow to be display.
        :rtype: bool
        """
        return self.__intro_show_license

    @intro_show_license.setter
    def intro_show_license(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError(
                "'show_license_line' property value must be a bool type or None"
            )
        if self.intro_show_license != value:
            self.__intro_show_license = value
        if (
            "license" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["license"] != value
        ):
            self.config.data["intro"]["show"]["license"] = value
            self.config.write_config()

    @property
    def intro_show_loader(self):
        """
        That property is use in front on the user space configuration file, and be like a storage value wrapper.

        If the value change , the configuration file will be rewrite with fresh information's.

        :return: If ``True`` the LOADER line is allow to be display.
        :rtype: bool
        """
        return self.__intro_show_loader

    @intro_show_loader.setter
    def intro_show_loader(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError(
                "'show_line_loader' property value must be a bool type or None"
            )
        if self.intro_show_loader != value:
            self.__intro_show_loader = value
        if (
            "loader" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["loader"] != value
        ):
            self.config.data["intro"]["show"]["loader"] = value
            self.config.write_config()

    @property
    def intro_show_memory_free(self):
        """
        That property is use in front on the user space configuration file, and be like a storage value wrapper.

        If the value change , the configuration file will be rewrite with fresh information's.

        :return: If ``True`` the MEMORY FREE line is allow to be display.
        :rtype: bool
        """
        return self.__intro_show_memory_free

    @intro_show_memory_free.setter
    def intro_show_memory_free(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError(
                "'show_line_memory_free' property value must be a bool type or None"
            )
        if self.intro_show_memory_free != value:
            self.__intro_show_memory_free = value
        if (
            "memory_free" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["memory_free"] != value
        ):
            self.config.data["intro"]["show"]["memory_free"] = value
            self.config.write_config()

    @property
    def intro_show_memory_total(self):
        """
        That property is use in front on the user space configuration file, and be like a storage value wrapper.

        If the value change , the configuration file will be rewrite with fresh information's.

        :return: If ``True`` the MEMORY TOTAL line is allowed to display.
        :rtype: bool
        """
        return self.__intro_show_memory_total

    @intro_show_memory_total.setter
    def intro_show_memory_total(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError(
                "'show_memory_total_line' property value must be a bool type or None"
            )
        if self.intro_show_memory_total != value:
            self.__intro_show_memory_total = value
        if (
            "memory_total" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["memory_total"] != value
        ):
            self.config.data["intro"]["show"]["memory_total"] = value
            self.config.write_config()

    @property
    def intro_show_rom(self):
        """
        That property is use in front on the user space configuration file, and be like a storage value wrapper.

        If the value change , the configuration file will be rewrite with fresh information's.

        :return: If ``True`` the ROM line is allowed to be display.
        :rtype: bool
        """
        return self.__intro_show_rom

    @intro_show_rom.setter
    def intro_show_rom(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError(
                "'show_line_rom' property value must be a bool type or None"
            )
        if self.intro_show_rom != value:
            self.__intro_show_rom = value
        if (
            "rom" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["rom"] != value
        ):
            self.config.data["intro"]["show"]["rom"] = value
            self.config.write_config()

    @property
    def intro_show_spacing(self):
        return self.__intro_show_spacing

    @intro_show_spacing.setter
    def intro_show_spacing(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError(
                "'intro_show_spacing' property value must be a bool type or None"
            )
        if self.intro_show_spacing != value:
            self.__intro_show_spacing = value
        if (
            "spacing" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["spacing"] != value
        ):
            self.config.data["intro"]["show"]["spacing"] = value
            self.config.write_config()

    @property
    def intro_show_title(self):
        """
        That property is use in front on the user space configuration file, and be like a storage value wrapper.

        If the value change , the configuration file will be rewrite with fresh information's.

        :return: If ``True`` the TITLE line is allowed to be display.
        :rtype: bool
        """
        return self.__intro_show_title

    @intro_show_title.setter
    def intro_show_title(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError(
                "'show_line_title' property value must be a bool type or None"
            )
        if self.intro_show_title != value:
            self.__intro_show_title = value
        if (
            "title" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["title"] != value
        ):
            self.config.data["intro"]["show"]["title"] = value
            self.config.write_config()

    def onchange_intro_show_exec(self, param_name, old_value, new_value):
        """Internal function call by ``cmd2`` python module during a ``set``"""
        pass

    def onchange_intro_show_holotape(self, param_name, old_value, new_value):
        """Internal function call by ``cmd2`` python module during a ``set``"""
        pass

    def onchange_intro_show_license(self, param_name, old_value, new_value):
        """Internal function call by ``cmd2`` python module during a ``set``"""
        pass

    def onchange_intro_show_loader(self, param_name, old_value, new_value):
        """Internal function call by ``cmd2`` python module during a ``set``"""
        pass

    def onchange_intro_show_memory_free(self, param_name, old_value, new_value):
        """Internal function call by ``cmd2`` python module during a ``set``"""
        pass

    def onchange_intro_show_memory_total(self, param_name, old_value, new_value):
        """Internal function call by ``cmd2`` python module during a ``set``"""
        pass

    def onchange_intro_show_rom(self, param_name, old_value, new_value):
        """Internal function call by ``cmd2`` python module during a ``set``"""
        pass

    def onchange_intro_show_spacing(self, param_name, old_value, new_value):
        """Internal function call by ``cmd2`` python module during a ``set``"""
        pass

    def onchange_intro_show_title(self, param_name, old_value, new_value):
        """Internal function call by ``cmd2`` python module during a ``set``"""
        pass

    @property
    def intro_to_display(self):
        """
        The function is use by the GLXShell class for get intro text to display.

        That is responsibility of GLXShellIntro class to know what display where GLXShell class only get the text.

        Example:

        .. code:: text

            ************************** SUPERSHELL V42.1 ****************************


            EXEC VENV PYTHON 3.7.3
            LOADER #1 SMP DEBIAN 4.19.146-1 (2020-09-17)
            31.36GB RAM SYSTEM
            20.85GB FREE
            LOAD ROM(1): DEITRIX 303

            (venv) me@localdomain:~/galaxie-shell $
            >

        :return: a big string it correspond to the entire intro to display
        :rtype: str
        """

        to_return = []

        if self.intro_show_title:
            to_return.append(self.intro_line_title)
            if self.intro_show_spacing:
                to_return.append("")
                to_return.append("")

        if self.intro_show_license:
            to_return.append(self.intro_line_license)

        if self.intro_show_loader:
            to_return.append(self.intro_line_loader)

        if self.intro_show_exec:
            to_return.append(self.intro_line_exec)

        if self.intro_show_memory_total:
            to_return.append(self.intro_line_memory_total)

        if self.intro_show_memory_free:
            to_return.append(self.intro_line_memory_free)

        if self.intro_show_holotape:
            to_return.append(self.intro_line_holotape)

        if self.intro_show_rom:
            to_return.append(self.intro_line_rom)

        if self.intro_show_spacing:
            to_return.append("")

        return "\n".join(to_return)



from GLXShell import APPLICATION_AUTHORS
from GLXShell import APPLICATION_DESCRIPTION
from GLXShell import APPLICATION_LICENSE
from GLXShell import APPLICATION_NAME
from GLXShell import APPLICATION_VERSION
from GLXShell import APPLICATION_WARRANTY


class GLXShPropertyApplication(object):
    def __init__(self):

        self.__authors = None
        self.__description = None
        self.__license = None
        self.__name = None
        self.__version = None
        self.__warranty = None

        self.authors = None
        self.description = None
        self.license = None
        self.name = None
        self.version = None
        self.warranty = None

    @property
    def authors(self):
        """
        it property store authors of the application.

        .. note:: if ``authors`` property value is set to None it return APPLICATION_AUTHORS from
                  Galaxie Shell __init__.py file.

        :return: the authors of the application
        :rtype: list
        """
        return self.__authors

    @authors.setter
    def authors(self, value=None):
        if value is None:
            value = APPLICATION_AUTHORS
        if type(value) != list:
            raise TypeError("'authors' property value must be a list or None")
        if self.authors != value:
            self.__authors = value

    @property
    def description(self):
        """
        it property store description of the application.

        .. note:: if ``description`` property value is set to None it return APPLICATION_DESCRIPTION from
                  Galaxie Shell __init__.py file.

        :return: the description of the application
        :rtype: str
        """
        return self.__description

    @description.setter
    def description(self, value=None):
        if value is None:
            value = APPLICATION_DESCRIPTION
        if type(value) != str:
            raise TypeError("'description' property value must be a str or None")
        if self.description != value:
            self.__description = value

    @property
    def license(self):
        """
        it property store license of the application.

        .. note:: if ``license`` property value is set to None it return APPLICATION_LICENSE from
                  Galaxie Shell __init__.py file.

        :return: the license of the application
        :rtype: str
        """
        return self.__license

    @license.setter
    def license(self, value=None):
        if value is None:
            value = APPLICATION_LICENSE
        if type(value) != str:
            raise TypeError("'license' property value must be a str or None")
        if self.license != value:
            self.__license = value

    @property
    def name(self):
        """
        it property store name of the application.

        .. note:: if ``name`` property value is set to None it return APPLICATION_NAME from
                  Galaxie Shell __init__.py file.

        :return: the name of the application
        :rtype: str
        """
        return self.__name

    @name.setter
    def name(self, value=None):
        if value is None:
            value = APPLICATION_NAME
        if type(value) != str:
            raise TypeError("'name' property value must be a str or None")
        if self.name != value:
            self.__name = value

    @property
    def version(self):
        """
        it property store version of the application.

        .. note:: if ``version`` property value is set to None it return APPLICATION_VERSION from
                  Galaxie Shell __init__.py file.

        :return: the version of the application
        :rtype: str
        """
        return self.__version

    @version.setter
    def version(self, value=None):
        if value is None:
            value = APPLICATION_VERSION
        if type(value) != str:
            raise TypeError("'version' property value must be a str or None")
        if self.version != value:
            self.__version = value

    @property
    def warranty(self):
        """
        it property store warranty of the application.

        .. note:: if ``warranty`` property value is set to None it return APPLICATION_WARRANTY from
                  Galaxie Shell __init__.py file.

        :return: the warranty of the application
        :rtype: str
        """
        return self.__warranty

    @warranty.setter
    def warranty(self, value=None):
        if value is None:
            value = APPLICATION_WARRANTY
        if type(value) != str:
            raise TypeError("'warranty' property value must be a str or None")
        if self.warranty != value:
            self.__warranty = value
